@@ -3,7 +3,7 @@
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #          Norman Barker <norman.barker@gmail.com>
 #
-# Copyright (c) 2020 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -28,7 +28,7 @@
 #
 # =================================================================
 
-""" Flask module providing the route paths to the api"""
+"""Flask module providing the route paths to the api"""
 
 import os
 
@@ -59,15 +59,18 @@ BLUEPRINT = Blueprint('pygeoapi', __name__, static_folder=STATIC_FOLDER)
 
 # CORS: optionally enable from config.
 if CONFIG['server'].get('cors', False):
-    from flask_cors import CORS
-    CORS(APP)
+    try:
+        from flask_cors import CORS
+        CORS(APP)
+    except ModuleNotFoundError:
+        print('Python package flask-cors required for CORS support')
 
 APP.config['JSONIFY_PRETTYPRINT_REGULAR'] = CONFIG['server'].get(
     'pretty_print', True)
 
 api_ = API(CONFIG)
 
-OGC_SCHEMAS_LOCATION = CONFIG['server'].get('ogc_schemas_location', None)
+OGC_SCHEMAS_LOCATION = CONFIG['server'].get('ogc_schemas_location')
 
 if (OGC_SCHEMAS_LOCATION is not None and
         not OGC_SCHEMAS_LOCATION.startswith('http')):
@@ -151,7 +154,7 @@ def conformance():
 
 
 @BLUEPRINT.route('/collections')
-@BLUEPRINT.route('/collections/<collection_id>')
+@BLUEPRINT.route('/collections/<path:collection_id>')
 def collections(collection_id=None):
     """
     OGC API collections endpoint
@@ -163,7 +166,7 @@ def collections(collection_id=None):
     return get_response(api_.describe_collections(request, collection_id))
 
 
-@BLUEPRINT.route('/collections/<collection_id>/queryables')
+@BLUEPRINT.route('/collections/<path:collection_id>/queryables')
 def collection_queryables(collection_id=None):
     """
     OGC API collections querybles endpoint
@@ -175,8 +178,10 @@ def collection_queryables(collection_id=None):
     return get_response(api_.get_collection_queryables(request, collection_id))
 
 
-@BLUEPRINT.route('/collections/<collection_id>/items', methods=['GET', 'POST'])
-@BLUEPRINT.route('/collections/<collection_id>/items/<item_id>')
+@BLUEPRINT.route('/collections/<path:collection_id>/items',
+                 methods=['GET', 'POST'])
+@BLUEPRINT.route('/collections/<path:collection_id>/items/<path:item_id>',
+                 methods=['GET', 'PUT', 'DELETE'])
 def collection_items(collection_id, item_id=None):
     """
     OGC API collections items endpoint
@@ -186,20 +191,35 @@ def collection_items(collection_id, item_id=None):
 
     :returns: HTTP response
     """
+
     if item_id is None:
         if request.method == 'GET':  # list items
             return get_response(
                 api_.get_collection_items(request, collection_id))
-        elif request.method == 'POST':  # filter items
-            return get_response(
-                api_.post_collection_items(request, collection_id))
+        elif request.method == 'POST':  # filter or manage items
+            if request.content_type is not None:
+                if request.content_type == 'application/geo+json':
+                    return get_response(
+                        api_.manage_collection_item(request, 'create',
+                                                    collection_id))
+                else:
+                    return get_response(
+                        api_.post_collection_items(request, collection_id))
 
+    elif request.method == 'DELETE':
+        return get_response(
+            api_.manage_collection_item(request, 'delete',
+                                        collection_id, item_id))
+    elif request.method == 'PUT':
+        return get_response(
+            api_.manage_collection_item(request, 'update',
+                                        collection_id, item_id))
     else:
         return get_response(
             api_.get_collection_item(request, collection_id, item_id))
 
 
-@BLUEPRINT.route('/collections/<collection_id>/coverage')
+@BLUEPRINT.route('/collections/<path:collection_id>/coverage')
 def collection_coverage(collection_id):
     """
     OGC API - Coverages coverage endpoint
@@ -211,7 +231,7 @@ def collection_coverage(collection_id):
     return get_response(api_.get_collection_coverage(request, collection_id))
 
 
-@BLUEPRINT.route('/collections/<collection_id>/coverage/domainset')
+@BLUEPRINT.route('/collections/<path:collection_id>/coverage/domainset')
 def collection_coverage_domainset(collection_id):
     """
     OGC API - Coverages coverage domainset endpoint
@@ -224,7 +244,7 @@ def collection_coverage_domainset(collection_id):
         request, collection_id))
 
 
-@BLUEPRINT.route('/collections/<collection_id>/coverage/rangetype')
+@BLUEPRINT.route('/collections/<path:collection_id>/coverage/rangetype')
 def collection_coverage_rangetype(collection_id):
     """
     OGC API - Coverages coverage rangetype endpoint
@@ -237,7 +257,7 @@ def collection_coverage_rangetype(collection_id):
         request, collection_id))
 
 
-@BLUEPRINT.route('/collections/<collection_id>/tiles')
+@BLUEPRINT.route('/collections/<path:collection_id>/tiles')
 def get_collection_tiles(collection_id=None):
     """
     OGC open api collections tiles access point
@@ -250,7 +270,8 @@ def get_collection_tiles(collection_id=None):
         request, collection_id))
 
 
-@BLUEPRINT.route('/collections/<collection_id>/tiles/<tileMatrixSetId>/metadata')  # noqa
+@BLUEPRINT.route('/collections/<path:collection_id>/tiles/<tileMatrixSetId>')
+@BLUEPRINT.route('/collections/<path:collection_id>/tiles/<tileMatrixSetId>/metadata')  # noqa
 def get_collection_tiles_metadata(collection_id=None, tileMatrixSetId=None):
     """
     OGC open api collection tiles service metadata
@@ -264,7 +285,7 @@ def get_collection_tiles_metadata(collection_id=None, tileMatrixSetId=None):
         request, collection_id, tileMatrixSetId))
 
 
-@BLUEPRINT.route('/collections/<collection_id>/tiles/\
+@BLUEPRINT.route('/collections/<path:collection_id>/tiles/\
 <tileMatrixSetId>/<tileMatrix>/<tileRow>/<tileCol>')
 def get_collection_tiles_data(collection_id=None, tileMatrixSetId=None,
                               tileMatrix=None, tileRow=None, tileCol=None):
@@ -283,8 +304,31 @@ def get_collection_tiles_data(collection_id=None, tileMatrixSetId=None,
         request, collection_id, tileMatrixSetId, tileMatrix, tileRow, tileCol))
 
 
+@BLUEPRINT.route('/collections/<collection_id>/map')
+@BLUEPRINT.route('/collections/<collection_id>/styles/<style_id>/map')
+def collection_map(collection_id, style_id=None):
+    """
+    OGC API - Maps map render endpoint
+
+    :param collection_id: collection identifier
+    :param style_id: style identifier
+
+    :returns: HTTP response
+    """
+
+    headers, status_code, content = api_.get_collection_map(
+        request, collection_id, style_id)
+
+    response = make_response(content, status_code)
+
+    if headers:
+        response.headers = headers
+
+    return response
+
+
 @BLUEPRINT.route('/processes')
-@BLUEPRINT.route('/processes/<process_id>')
+@BLUEPRINT.route('/processes/<path:process_id>')
 def get_processes(process_id=None):
     """
     OGC API - Processes description endpoint
@@ -296,30 +340,28 @@ def get_processes(process_id=None):
     return get_response(api_.describe_processes(request, process_id))
 
 
-@BLUEPRINT.route('/processes/<process_id>/jobs')
-@BLUEPRINT.route('/processes/<process_id>/jobs/<job_id>',
+@BLUEPRINT.route('/jobs')
+@BLUEPRINT.route('/jobs/<job_id>',
                  methods=['GET', 'DELETE'])
-def get_process_jobs(process_id=None, job_id=None):
+def get_jobs(job_id=None):
     """
     OGC API - Processes jobs endpoint
 
-    :param process_id: process identifier
     :param job_id: job identifier
 
     :returns: HTTP response
     """
 
     if job_id is None:
-        return get_response(api_.get_process_jobs(request, process_id))
+        return get_response(api_.get_jobs(request))
     else:
         if request.method == 'DELETE':  # dismiss job
-            return get_response(api_.delete_process_job(process_id, job_id))
+            return get_response(api_.delete_job(job_id))
         else:  # Return status of a specific job
-            return get_response(api_.get_process_jobs(
-                request, process_id, job_id))
+            return get_response(api_.get_jobs(request, job_id))
 
 
-@BLUEPRINT.route('/processes/<process_id>/execution', methods=['POST'])
+@BLUEPRINT.route('/processes/<path:process_id>/execution', methods=['POST'])
 def execute_process_jobs(process_id):
     """
     OGC API - Processes execution endpoint
@@ -332,47 +374,44 @@ def execute_process_jobs(process_id):
     return get_response(api_.execute_process(request, process_id))
 
 
-@BLUEPRINT.route('/processes/<process_id>/jobs/<job_id>/results',
+@BLUEPRINT.route('/jobs/<job_id>/results',
                  methods=['GET'])
-def get_process_job_result(process_id=None, job_id=None):
+def get_job_result(job_id=None):
     """
     OGC API - Processes job result endpoint
 
-    :param process_id: process identifier
     :param job_id: job identifier
 
     :returns: HTTP response
     """
-    return get_response(api_.get_process_job_result(
-        request, process_id, job_id))
+    return get_response(api_.get_job_result(request, job_id))
 
 
-@BLUEPRINT.route('/processes/<process_id>/jobs/<job_id>/results/<resource>',
+@BLUEPRINT.route('/jobs/<job_id>/results/<resource>',
                  methods=['GET'])
-def get_process_job_result_resource(process_id, job_id, resource):
+def get_job_result_resource(job_id, resource):
     """
     OGC API - Processes job result resource endpoint
 
-    :param process_id: process identifier
     :param job_id: job identifier
     :param resource: job resource
 
     :returns: HTTP response
     """
-    return get_response(api_.get_process_job_result_resource(
-        request, process_id, job_id, resource))
+    return get_response(api_.get_job_result_resource(
+        request, job_id, resource))
 
 
-@BLUEPRINT.route('/collections/<collection_id>/position')
-@BLUEPRINT.route('/collections/<collection_id>/area')
-@BLUEPRINT.route('/collections/<collection_id>/cube')
-@BLUEPRINT.route('/collections/<collection_id>/trajectory')
-@BLUEPRINT.route('/collections/<collection_id>/corridor')
-@BLUEPRINT.route('/collections/<collection_id>/instances/<instance_id>/position')  # noqa
-@BLUEPRINT.route('/collections/<collection_id>/instances/<instance_id>/area')
-@BLUEPRINT.route('/collections/<collection_id>/instances/<instance_id>/cube')
-@BLUEPRINT.route('/collections/<collection_id>/instances/<instance_id>/trajectory')  # noqa
-@BLUEPRINT.route('/collections/<collection_id>/instances/<instance_id>/corridor')  # noqa
+@BLUEPRINT.route('/collections/<path:collection_id>/position')
+@BLUEPRINT.route('/collections/<path:collection_id>/area')
+@BLUEPRINT.route('/collections/<path:collection_id>/cube')
+@BLUEPRINT.route('/collections/<path:collection_id>/trajectory')
+@BLUEPRINT.route('/collections/<path:collection_id>/corridor')
+@BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/position')  # noqa
+@BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/area')  # noqa
+@BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/cube')  # noqa
+@BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/trajectory')  # noqa
+@BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/corridor')  # noqa
 def get_collection_edr_query(collection_id, instance_id=None):
     """
     OGC EDR API endpoints
