@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2018 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -32,7 +32,7 @@ import logging
 
 import unicodecsv as csv
 
-from pygeoapi.formatter.base import BaseFormatter
+from pygeoapi.formatter.base import BaseFormatter, FormatterSerializationError
 
 LOGGER = logging.getLogger(__name__)
 
@@ -40,7 +40,7 @@ LOGGER = logging.getLogger(__name__)
 class CSVFormatter(BaseFormatter):
     """CSV formatter"""
 
-    def __init__(self, formatter_def):
+    def __init__(self, formatter_def: dict):
         """
         Initialize object
 
@@ -56,7 +56,7 @@ class CSVFormatter(BaseFormatter):
         super().__init__({'name': 'csv', 'geom': geom})
         self.mimetype = 'text/csv'
 
-    def write(self, options={}, data=None):
+    def write(self, options: dict = {}, data: dict = None) -> str:
         """
         Generate data in CSV format
 
@@ -83,20 +83,25 @@ class CSVFormatter(BaseFormatter):
                 # TODO: implement wkt geometry serialization
                 LOGGER.debug('not a point geometry, skipping')
 
-        LOGGER.debug('CSV fields: {}'.format(fields))
+        LOGGER.debug(f'CSV fields: {fields}')
 
-        output = io.BytesIO()
-        writer = csv.DictWriter(output, fields)
-        writer.writeheader()
+        try:
+            output = io.BytesIO()
+            writer = csv.DictWriter(output, fields)
+            writer.writeheader()
+
+            for feature in data['features']:
+                fp = feature['properties']
+                if is_point:
+                    fp['x'] = feature['geometry']['coordinates'][0]
+                    fp['y'] = feature['geometry']['coordinates'][1]
+                LOGGER.debug(fp)
+                writer.writerow(fp)
+        except ValueError as err:
+            LOGGER.error(err)
+            raise FormatterSerializationError('Error writing CSV output')
 
-        for feature in data['features']:
-            fp = feature['properties']
-            if is_point:
-                fp['x'] = feature['geometry']['coordinates'][0]
-                fp['y'] = feature['geometry']['coordinates'][1]
-            LOGGER.debug(fp)
-            writer.writerow(fp)
         return output.getvalue()
 
     def __repr__(self):
-        return '<CSVFormatter> {}'.format(self.mimetype)
+        return f'<CSVFormatter> {self.name}'
