@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2020 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -28,13 +28,14 @@
 # =================================================================
 
 from datetime import datetime
-import io
 import json
 import logging
 from multiprocessing import dummy
-import os
+from pathlib import Path
+from typing import Any, Tuple
 
 from pygeoapi.util import DATETIME_FORMAT, JobStatus
+from pygeoapi.process.base import BaseProcessor
 
 LOGGER = logging.getLogger(__name__)
 
@@ -42,7 +43,7 @@ LOGGER = logging.getLogger(__name__)
 class BaseManager:
     """generic Manager ABC"""
 
-    def __init__(self, manager_def):
+    def __init__(self, manager_def: dict):
         """
         Initialize object
 
@@ -53,14 +54,16 @@ class BaseManager:
 
         self.name = manager_def['name']
         self.is_async = False
-        self.connection = manager_def.get('connection', None)
-        self.output_dir = manager_def.get('output_dir', None)
+        self.connection = manager_def.get('connection')
+        self.output_dir = manager_def.get('output_dir')
 
-    def get_jobs(self, process_id=None, status=None):
+        if self.output_dir is not None:
+            self.output_dir = Path(self.output_dir)
+
+    def get_jobs(self, status: JobStatus = None) -> list:
         """
         Get process jobs, optionally filtered by status
 
-        :param process_id: process identifier
         :param status: job status (accepted, running, successful,
                        failed, results) (default is all)
 
@@ -69,7 +72,7 @@ class BaseManager:
 
         raise NotImplementedError()
 
-    def add_job(self, job_metadata):
+    def add_job(self, job_metadata: dict) -> str:
         """
         Add a job
 
@@ -80,11 +83,10 @@ class BaseManager:
 
         raise NotImplementedError()
 
-    def update_job(self, process_id, job_id, update_dict):
+    def update_job(self, job_id: str, update_dict: dict) -> bool:
         """
         Updates a job
 
-        :param process_id: process identifier
         :param job_id: job identifier
         :param update_dict: `dict` of property updates
 
@@ -93,11 +95,10 @@ class BaseManager:
 
         raise NotImplementedError()
 
-    def get_job(self, process_id, job_id):
+    def get_job(self, job_id: str) -> dict:
         """
         Get a job (!)
 
-        :param process_id: process identifier
         :param job_id: job identifier
 
         :returns: `dict` of job result
@@ -105,11 +106,10 @@ class BaseManager:
 
         raise NotImplementedError()
 
-    def get_job_result(self, process_id, job_id):
+    def get_job_result(self, job_id: str) -> Tuple[str, Any]:
         """
         Returns the actual output from a completed process
 
-        :param process_id: process identifier
         :param job_id: job identifier
 
         :returns: `tuple` of mimetype and raw output
@@ -117,11 +117,10 @@ class BaseManager:
 
         raise NotImplementedError()
 
-    def delete_job(self, process_id, job_id):
+    def delete_job(self, job_id: str) -> bool:
         """
         Deletes a job and associated results/outputs
 
-        :param process_id: process identifier
         :param job_id: job identifier
 
         :returns: `bool` of status result
@@ -129,7 +128,8 @@ class BaseManager:
 
         raise NotImplementedError()
 
-    def _execute_handler_async(self, p, job_id, data_dict):
+    def _execute_handler_async(self, p: BaseProcessor, job_id: str,
+                               data_dict: dict) -> Tuple[None, JobStatus]:
         """
         This private execution handler executes a process in a background
         thread using `multiprocessing.dummy`
@@ -150,7 +150,8 @@ class BaseManager:
         _process.start()
         return 'application/json', None, JobStatus.accepted
 
-    def _execute_handler_sync(self, p, job_id, data_dict):
+    def _execute_handler_sync(self, p: BaseProcessor, job_id: str,
+                              data_dict: dict) -> Tuple[str, Any, JobStatus]:
         """
         Synchronous execution handler
 
@@ -185,24 +186,32 @@ class BaseManager:
 
         try:
             if self.output_dir is not None:
-                filename = '{}-{}'.format(p.metadata['id'], job_id)
-                job_filename = os.path.join(self.output_dir, filename)
+                filename = f"{p.metadata['id']}-{job_id}"
+                job_filename = self.output_dir / filename
             else:
                 job_filename = None
 
             current_status = JobStatus.running
             jfmt, outputs = p.execute(data_dict)
 
-            self.update_job(process_id, job_id, {
+            self.update_job(job_id, {
                 'status': current_status.value,
                 'message': 'Writing job output',
                 'progress': 95
             })
 
             if self.output_dir is not None:
-                LOGGER.debug('writing output to {}'.format(job_filename))
-                with io.open(job_filename, 'w', encoding='utf-8') as fh:
-                    fh.write(json.dumps(outputs, sort_keys=True, indent=4))
+                LOGGER.debug(f'writing output to {job_filename}')
+                if isinstance(outputs, dict):
+                    mode = 'w'
+                    data = json.dumps(outputs, sort_keys=True, indent=4)
+                    encoding = 'utf-8'
+                elif isinstance(outputs, bytes):
+                    mode = 'wb'
+                    data = outputs
+                    encoding = None
+                with job_filename.open(mode=mode, encoding=encoding) as fh:
+                    fh.write(data)
 
             current_status = JobStatus.successful
 
@@ -210,13 +219,13 @@ class BaseManager:
                 'job_end_datetime': datetime.utcnow().strftime(
                     DATETIME_FORMAT),
                 'status': current_status.value,
-                'location': job_filename,
+                'location': str(job_filename),
                 'mimetype': jfmt,
                 'message': 'Job complete',
                 'progress': 100
             }
 
-            self.update_job(process_id, job_id, job_update_metadata)
+            self.update_job(job_id, job_update_metadata)
 
         except Exception as err:
             # TODO assess correct exception type and description to help users
@@ -227,6 +236,7 @@ class BaseManager:
             # endpoint, even if the /result endpoint correctly returns the
             # failure information (i.e. what one might assume is a 200
             # response).
+
             current_status = JobStatus.failed
             code = 'InvalidParameterValue'
             outputs = {
@@ -245,7 +255,7 @@ class BaseManager:
 
             jfmt = 'application/json'
 
-            self.update_job(process_id, job_id, job_metadata)
+            self.update_job(job_id, job_metadata)
 
         return jfmt, outputs, current_status
 
@@ -269,4 +279,4 @@ class BaseManager:
             return self._execute_handler_async(p, job_id, data_dict)
 
     def __repr__(self):
-        return '<BaseManager> {}'.format(self.name)
+        return f'<BaseManager> {self.name}'
