@@ -27,10 +27,11 @@
 #
 # =================================================================
 
-import io
+import fcntl
 import json
 import logging
-import os
+from pathlib import Path
+from typing import Any, Tuple
 
 import tinydb
 
@@ -43,7 +44,7 @@ LOGGER = logging.getLogger(__name__)
 class TinyDBManager(BaseManager):
     """TinyDB Manager"""
 
-    def __init__(self, manager_def):
+    def __init__(self, manager_def: dict):
         """
         Initialize object
 
@@ -55,8 +56,7 @@ class TinyDBManager(BaseManager):
         super().__init__(manager_def)
         self.is_async = True
 
-    def _connect(self):
-
+    def _connect(self, mode: str = 'r') -> bool:
         """
         connect to manager
 
@@ -64,9 +64,13 @@ class TinyDBManager(BaseManager):
         """
 
         self.db = tinydb.TinyDB(self.connection)
+
+        if mode == 'w':
+            fcntl.lockf(self.db.storage._handle, fcntl.LOCK_EX)
+
         return True
 
-    def destroy(self):
+    def destroy(self) -> bool:
         """
         Destroy manager
 
@@ -77,7 +81,7 @@ class TinyDBManager(BaseManager):
         self.db.close()
         return True
 
-    def get_jobs(self, status=None):
+    def get_jobs(self, status: JobStatus = None) -> list:
         """
         Get jobs
 
@@ -93,7 +97,7 @@ class TinyDBManager(BaseManager):
 
         return jobs_list
 
-    def add_job(self, job_metadata):
+    def add_job(self, job_metadata: dict) -> str:
         """
         Add a job
 
@@ -102,13 +106,13 @@ class TinyDBManager(BaseManager):
         :returns: identifier of added job
         """
 
-        self._connect()
+        self._connect(mode='w')
         doc_id = self.db.insert(job_metadata)
         self.db.close()
 
         return doc_id
 
-    def update_job(self, job_id, update_dict):
+    def update_job(self, job_id: str, update_dict: dict) -> bool:
         """
         Updates a job
 
@@ -118,13 +122,13 @@ class TinyDBManager(BaseManager):
         :returns: `bool` of status result
         """
 
-        self._connect()
+        self._connect(mode='w')
         self.db.update(update_dict, tinydb.where('identifier') == job_id)
         self.db.close()
 
         return True
 
-    def delete_job(self, job_id):
+    def delete_job(self, job_id: str) -> bool:
         """
         Deletes a job
 
@@ -135,17 +139,17 @@ class TinyDBManager(BaseManager):
         # delete result file if present
         job_result = self.get_job(job_id)
         if job_result:
-            location = job_result.get('location', None)
+            location = job_result.get('location')
             if location and self.output_dir is not None:
-                os.remove(location)
+                Path(location).unlink()
 
-        self._connect()
+        self._connect(mode='w')
         removed = bool(self.db.remove(tinydb.where('identifier') == job_id))
         self.db.close()
 
         return removed
 
-    def get_job(self, job_id):
+    def get_job(self, job_id: str) -> dict:
         """
         Get a single job
 
@@ -162,7 +166,7 @@ class TinyDBManager(BaseManager):
         self.db.close()
         return result
 
-    def get_job_result(self, job_id):
+    def get_job_result(self, job_id: str) -> Tuple[str, Any]:
         """
         Get a job's status, and actual output of executing the process
 
@@ -176,8 +180,8 @@ class TinyDBManager(BaseManager):
             # job does not exist
             return None
 
-        location = job_result.get('location', None)
-        mimetype = job_result.get('mimetype', None)
+        location = job_result.get('location')
+        mimetype = job_result.get('mimetype')
         job_status = JobStatus[job_result['status']]
 
         if not job_status == JobStatus.successful:
@@ -187,11 +191,13 @@ class TinyDBManager(BaseManager):
             # Job data was not written for some reason
             # TODO log/raise exception?
             return (None,)
+        else:
+            location = Path(location)
 
-        with io.open(location, 'r', encoding='utf-8') as filehandler:
+        with location.open('r', encoding='utf-8') as filehandler:
             result = json.load(filehandler)
 
         return mimetype, result
 
     def __repr__(self):
-        return '<TinyDBManager> {}'.format(self.name)
+        return f'<TinyDBManager> {self.name}'
