@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2021 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -34,6 +34,7 @@ import logging
 
 from pygeoapi.provider.base import (BaseProvider, ProviderQueryError,
                                     ProviderItemNotFoundError)
+from pygeoapi.util import get_typed_value
 
 LOGGER = logging.getLogger(__name__)
 
@@ -67,17 +68,34 @@ class CSVProvider(BaseProvider):
             LOGGER.debug('Serializing DictReader')
             data_ = csv.DictReader(ff)
             fields = {}
-            for f in data_.fieldnames:
-                fields[f] = {'type': 'string'}
+
+            row = next(data_)
+
+            for key, value in row.items():
+                LOGGER.debug(f'key: {key}, value: {value}')
+                value2 = get_typed_value(value)
+                if key in [self.geometry_x, self.geometry_y]:
+                    continue
+                if key == self.id_field:
+                    type_ = 'string'
+                elif isinstance(value2, float):
+                    type_ = 'number'
+                elif isinstance(value2, int):
+                    type_ = 'integer'
+                else:
+                    type_ = 'string'
+
+                fields[key] = {'type': type_}
+
             return fields
 
-    def _load(self, startindex=0, limit=10, resulttype='results',
+    def _load(self, offset=0, limit=10, resulttype='results',
               identifier=None, bbox=[], datetime_=None, properties=[],
               select_properties=[], skip_geometry=False, q=None):
         """
         Load CSV data
 
-        :param startindex: starting record to return (default 0)
+        :param offset: starting record to return (default 0)
         :param limit: number of records to return (default 10)
         :param datetime_: temporal (datestamp or extent)
         :param resulttype: return results or hit limit (default results)
@@ -109,34 +127,46 @@ class CSVProvider(BaseProvider):
                 feature_collection['numberMatched'] = len(list(data_))
                 return feature_collection
             LOGGER.debug('Slicing CSV rows')
-            for row in itertools.islice(data_, startindex, startindex+limit):
+            for row in itertools.islice(data_, offset, offset+limit):
+                try:
+                    coordinates = [
+                        float(row.pop(self.geometry_x)),
+                        float(row.pop(self.geometry_y)),
+                    ]
+                except ValueError:
+                    msg = f'Skipping row with invalid geometry: {row.get(self.id_field)}'  # noqa
+                    LOGGER.error(msg)
+                    continue
                 feature = {'type': 'Feature'}
                 feature['id'] = row.pop(self.id_field)
                 if not skip_geometry:
                     feature['geometry'] = {
                         'type': 'Point',
-                        'coordinates': [
-                            float(row.pop(self.geometry_x)),
-                            float(row.pop(self.geometry_y))
-                        ]
+                        'coordinates': coordinates
                     }
                 else:
                     feature['geometry'] = None
+
+                feature['properties'] = OrderedDict()
+
                 if self.properties or select_properties:
-                    feature['properties'] = OrderedDict()
                     for p in set(self.properties) | set(select_properties):
                         try:
-                            feature['properties'][p] = row[p]
+                            feature['properties'][p] = get_typed_value(row[p])
                         except KeyError as err:
                             LOGGER.error(err)
                             raise ProviderQueryError()
                 else:
-                    feature['properties'] = row
+                    for key, value in row.items():
+                        LOGGER.debug(f'key: {key}, value: {value}')
+                        feature['properties'][key] = get_typed_value(value)
 
                 if identifier is not None and feature['id'] == identifier:
                     found = True
                     result = feature
+
                 feature_collection['features'].append(feature)
+
                 feature_collection['numberMatched'] = \
                     len(feature_collection['features'])
 
@@ -150,13 +180,13 @@ class CSVProvider(BaseProvider):
 
         return feature_collection
 
-    def query(self, startindex=0, limit=10, resulttype='results',
+    def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
               select_properties=[], skip_geometry=False, q=None, **kwargs):
         """
         CSV query
 
-        :param startindex: starting record to return (default 0)
+        :param offset: starting record to return (default 0)
         :param limit: number of records to return (default 10)
         :param resulttype: return results or hit limit (default results)
         :param bbox: bounding box [minx,miny,maxx,maxy]
@@ -170,7 +200,7 @@ class CSVProvider(BaseProvider):
         :returns: dict of GeoJSON FeatureCollection
         """
 
-        return self._load(startindex, limit, resulttype,
+        return self._load(offset, limit, resulttype,
                           properties=properties,
                           select_properties=select_properties,
                           skip_geometry=skip_geometry)
@@ -187,9 +217,9 @@ class CSVProvider(BaseProvider):
         if item:
             return item
         else:
-            err = 'item {} not found'.format(identifier)
+            err = f'item {identifier} not found'
             LOGGER.error(err)
             raise ProviderItemNotFoundError(err)
 
     def __repr__(self):
-        return '<CSVProvider> {}'.format(self.data)
+        return f'<CSVProvider> {self.data}'
