@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2021 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 # Copyright (c) 2021 Francesco Bartoli
 #
 # Permission is hereby granted, free of charge, to any person
@@ -67,12 +67,14 @@ class ElasticsearchProvider(BaseProvider):
         LOGGER.debug('Setting Elasticsearch properties')
         self.is_gdal = False
 
-        LOGGER.debug('host: {}'.format(self.es_host))
-        LOGGER.debug('index: {}'.format(self.index_name))
+        LOGGER.debug(f'host: {self.es_host}')
+        LOGGER.debug(f'index: {self.index_name}')
 
         self.type_name = 'FeatureCollection'
         self.url_parsed = urlparse(self.es_host)
 
+        self.select_properties = []
+
         LOGGER.debug('Connecting to Elasticsearch')
 
         if self.url_parsed.port is None:  # proxy to default HTTP(S) port
@@ -92,7 +94,7 @@ class ElasticsearchProvider(BaseProvider):
         if self.url_parsed.path:
             url_settings['url_prefix'] = self.url_parsed.path
 
-        LOGGER.debug('URL settings: {}'.format(url_settings))
+        LOGGER.debug(f'URL settings: {url_settings}')
         LOGGER.debug('Connecting to Elasticsearch')
         self.es = Elasticsearch([url_settings])
         if not self.es.ping():
@@ -123,7 +125,7 @@ class ElasticsearchProvider(BaseProvider):
 
         fields_ = {}
         ic = IndicesClient(self.es)
-        ii = ic.get(self.index_name)
+        ii = ic.get(index=self.index_name)
 
         try:
             if '*' not in self.index_name:
@@ -136,6 +138,9 @@ class ElasticsearchProvider(BaseProvider):
             LOGGER.debug('ES index looks generated by GDAL')
             self.is_gdal = True
             p = ii[self.index_name]['mappings']
+        except IndexError:
+            LOGGER.warning('could not get fields; returning empty set')
+            return {}
 
         for k, v in p['properties'].items():
             if 'type' in v:
@@ -143,19 +148,21 @@ class ElasticsearchProvider(BaseProvider):
                     fields_[k] = {'type': 'string'}
                 elif v['type'] == 'date':
                     fields_[k] = {'type': 'string', 'format': 'date'}
+                elif v['type'] in ('float', 'long'):
+                    fields_[k] = {'type': 'number', 'format': v['type']}
                 else:
                     fields_[k] = {'type': v['type']}
 
         return fields_
 
-    def query(self, startindex=0, limit=10, resulttype='results',
+    def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
               select_properties=[], skip_geometry=False, q=None,
               filterq=None, **kwargs):
         """
         query Elasticsearch index
 
-        :param startindex: starting record to return (default 0)
+        :param offset: starting record to return (default 0)
         :param limit: number of records to return (default 10)
         :param resulttype: return results or hit limit (default results)
         :param bbox: bounding box [minx,miny,maxx,maxy]
@@ -170,6 +177,8 @@ class ElasticsearchProvider(BaseProvider):
         :returns: dict of 0..n GeoJSON features
         """
 
+        self.select_properties = select_properties
+
         query = {'track_total_hits': True, 'query': {'bool': {'filter': []}}}
         filter_ = []
 
@@ -253,13 +262,14 @@ class ElasticsearchProvider(BaseProvider):
             LOGGER.debug('processing sortby')
             query['sort'] = []
             for sort in sortby:
-                LOGGER.debug('processing sort object: {}'.format(sort))
+                LOGGER.debug(f'processing sort object: {sort}')
 
                 sp = sort['property']
 
-                if self.fields[sp]['type'] == 'string':
+                if (self.fields[sp]['type'] == 'string'
+                        and self.fields[sp].get('format') != 'date'):
                     LOGGER.debug('setting ES .raw on property')
-                    sort_property = '{}.raw'.format(self.mask_prop(sp))
+                    sort_property = f'{self.mask_prop(sp)}.raw'
                 else:
                     sort_property = self.mask_prop(sp)
 
@@ -286,19 +296,21 @@ class ElasticsearchProvider(BaseProvider):
                 ]
             }
 
-        if self.properties or select_properties:
-            LOGGER.debug('including specified fields: {}'.format(
-                self.properties))
+        if self.properties or self.select_properties:
+            LOGGER.debug('filtering properties')
+
+            all_properties = self.get_properties()
+
             query['_source'] = {
-                'includes': list(map(self.mask_prop,
-                                 set(self.properties) | set(select_properties)))  # noqa
+                'includes': list(map(self.mask_prop, all_properties))
             }
+
             query['_source']['includes'].append(self.mask_prop(self.id_field))
             query['_source']['includes'].append('type')
             query['_source']['includes'].append('geometry')
+
         if skip_geometry:
-            LOGGER.debug('limiting to specified fields: {}'.format(
-                select_properties))
+            LOGGER.debug('excluding geometry')
             try:
                 query['_source']['excludes'] = ['geometry']
             except KeyError:
@@ -306,34 +318,29 @@ class ElasticsearchProvider(BaseProvider):
         try:
             LOGGER.debug('querying Elasticsearch')
             if filterq:
-                LOGGER.debug('adding cql object: {}'.format(filterq.json()))
+                LOGGER.debug(f'adding cql object: {filterq.json()}')
                 query = update_query(input_query=query, cql=filterq)
             LOGGER.debug(json.dumps(query, indent=4))
 
-            LOGGER.debug('Setting ES paging zero-based')
-            if startindex > 0:
-                startindex2 = startindex - 1
-            else:
-                startindex2 = startindex
-
-            if startindex2 + limit > 10000:
+            LOGGER.debug('Testing for ES scrolling')
+            if offset + limit > 10000:
                 gen = helpers.scan(client=self.es, query=query,
                                    preserve_order=True,
                                    index=self.index_name)
                 results = {'hits': {'total': limit, 'hits': []}}
-                for i in range(startindex2 + limit):
+                for i in range(offset + limit):
                     try:
-                        if i >= startindex2:
+                        if i >= offset:
                             results['hits']['hits'].append(next(gen))
                         else:
                             next(gen)
                     except StopIteration:
                         break
                 results['hits']['total'] = \
-                    len(results['hits']['hits']) + startindex2
+                    len(results['hits']['hits']) + offset
             else:
                 results = self.es.search(index=self.index_name,
-                                         from_=startindex2, size=limit,
+                                         from_=offset, size=limit,
                                          body=query)
                 results['hits']['total'] = results['hits']['total']['value']
 
@@ -371,12 +378,12 @@ class ElasticsearchProvider(BaseProvider):
         """
 
         try:
-            LOGGER.debug('Fetching identifier {}'.format(identifier))
-            result = self.es.get(self.index_name, id=identifier)
+            LOGGER.debug(f'Fetching identifier {identifier}')
+            result = self.es.get(index=self.index_name, id=identifier)
             LOGGER.debug('Serializing feature')
             feature_ = self.esdoc2geojson(result)
         except exceptions.NotFoundError as err:
-            LOGGER.debug('Not found via ES id query: {}'.format(err))
+            LOGGER.debug(f'Not found via ES id query: {err}')
             LOGGER.debug('Trying via a real query')
 
             query = {
@@ -407,6 +414,55 @@ class ElasticsearchProvider(BaseProvider):
 
         return feature_
 
+    def create(self, item):
+        """
+        Create a new item
+
+        :param item: `dict` of new item
+
+        :returns: identifier of created item
+        """
+
+        identifier, json_data = self._load_and_prepare_item(item)
+
+        LOGGER.debug(f'Inserting data with identifier {identifier}')
+        _ = self.es.index(index=self.index_name, id=identifier, body=json_data)
+        LOGGER.debug('Item added')
+
+        return identifier
+
+    def update(self, identifier, item):
+        """
+        Updates an existing item
+
+        :param identifier: feature id
+        :param item: `dict` of partial or full item
+
+        :returns: `bool` of update result
+        """
+
+        LOGGER.debug(f'Updating item {identifier}')
+        identifier, json_data = self._load_and_prepare_item(
+            item, identifier, raise_if_exists=False)
+
+        _ = self.es.index(index=self.index_name, id=identifier, body=json_data)
+
+        return True
+
+    def delete(self, identifier):
+        """
+        Deletes an existing item
+
+        :param identifier: item id
+
+        :returns: `bool` of deletion result
+        """
+
+        LOGGER.debug(f'Deleting item {identifier}')
+        _ = self.es.delete(index=self.index_name, id=identifier)
+
+        return True
+
     def esdoc2geojson(self, doc):
         """
         generate GeoJSON `dict` from ES document
@@ -438,14 +494,17 @@ class ElasticsearchProvider(BaseProvider):
             feature_['id'] = id_
             feature_['geometry'] = doc['_source'].get('geometry')
 
-        if self.properties:
+        if self.properties or self.select_properties:
+            LOGGER.debug('Filtering properties')
+            all_properties = self.get_properties()
+
             feature_thinned = {
                 'id': id_,
                 'type': feature_['type'],
                 'geometry': feature_.get('geometry'),
                 'properties': OrderedDict()
             }
-            for p in self.properties:
+            for p in all_properties:
                 try:
                     feature_thinned['properties'][p] = feature_['properties'][p]  # noqa
                 except KeyError as err:
@@ -469,10 +528,26 @@ class ElasticsearchProvider(BaseProvider):
         if self.is_gdal:
             return property_name
         else:
-            return 'properties.{}'.format(property_name)
+            return f'properties.{property_name}'
+
+    def get_properties(self):
+        all_properties = []
+
+        LOGGER.debug(f'configured properties: {self.properties}')
+        LOGGER.debug(f'selected properties: {self.select_properties}')
+
+        if not self.properties and not self.select_properties:
+            all_properties = self.get_fields()
+        if self.properties and self.select_properties:
+            all_properties = set(self.properties) & set(self.select_properties)
+        else:
+            all_properties = set(self.properties) | set(self.select_properties)
+
+        LOGGER.debug(f'resulting properties: {all_properties}')
+        return all_properties
 
     def __repr__(self):
-        return '<ElasticsearchProvider> {}'.format(self.data)
+        return f'<ElasticsearchProvider> {self.data}'
 
 
 class ElasticsearchCatalogueProvider(ElasticsearchProvider):
@@ -489,18 +564,20 @@ class ElasticsearchCatalogueProvider(ElasticsearchProvider):
     def get_fields(self):
         fields = super().get_fields()
         for i in self._excludes():
-            del fields[i]
+            if i in fields:
+                del fields[i]
 
         fields['q'] = {'type': 'string'}
 
         return fields
 
-    def query(self, startindex=0, limit=10, resulttype='results',
+    def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
-              select_properties=[], skip_geometry=False, q=None):
+              select_properties=[], skip_geometry=False, q=None,
+              filterq=None, **kwargs):
 
         records = super().query(
-            startindex=startindex, limit=limit,
+            offset=offset, limit=limit,
             resulttype=resulttype, bbox=bbox,
             datetime_=datetime_, properties=properties,
             sortby=sortby,
@@ -511,7 +588,7 @@ class ElasticsearchCatalogueProvider(ElasticsearchProvider):
         return records
 
     def __repr__(self):
-        return '<ElasticsearchCatalogueProvider> {}'.format(self.data)
+        return f'<ElasticsearchCatalogueProvider> {self.data}'
 
 
 class ESQueryBuilder:
@@ -692,7 +769,5 @@ def update_query(input_query: Dict, cql: CQLModel):
     output_query = _build_query(query, cql)
     s = s.query(output_query)
 
-    LOGGER.debug('Enhanced query: {}'.format(
-        json.dumps(s.to_dict())
-    ))
+    LOGGER.debug(f'Enhanced query: {json.dumps(s.to_dict())}')
     return s.to_dict()
