@@ -3,7 +3,7 @@
 # Authors: Timo Tuunanen <timo.tuunanen@rdvelho.com>
 #
 # Copyright (c) 2019 Timo Tuunanen
-# Copyright (c) 2021 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -60,12 +60,13 @@ class MongoProvider(BaseProvider):
 
         super().__init__(provider_def)
 
-        LOGGER.info('Mongo source config: {}'.format(self.data))
+        LOGGER.info(f'Mongo source config: {self.data}')
 
         dbclient = MongoClient(self.data)
         self.featuredb = dbclient.get_default_database()
         self.collection = provider_def['collection']
         self.featuredb[self.collection].create_index([("geometry", GEOSPHERE)])
+        self.fields = self.get_fields()
 
     def get_fields(self):
         """
@@ -73,15 +74,25 @@ class MongoProvider(BaseProvider):
 
         :returns: dict of fields
         """
+
         map = Code(
             "function() { for (var key in this.properties) "
             "{ emit(key, null); } }")
         reduce = Code("function(key, stuff) { return null; }")
         result = self.featuredb[self.collection].map_reduce(
             map, reduce, "myresults")
-        return result.distinct('_id')
 
-    def _get_feature_list(self, filterObj, sortList=[], skip=0, maxitems=1):
+        # prepare a dictionary with fields
+        # set the field type to 'string'.
+        # by operating without a schema, mongo can query any data type.
+        fields = {}
+        for i in result.distinct('_id'):
+            fields[i] = {'type': 'string'}
+
+        return (fields)
+
+    def _get_feature_list(self, filterObj, sortList=[], skip=0, maxitems=1,
+                          skip_geometry=False):
         featurecursor = self.featuredb[self.collection].find(filterObj)
 
         if sortList:
@@ -93,12 +104,14 @@ class MongoProvider(BaseProvider):
         featurelist = list(featurecursor)
         for item in featurelist:
             item['id'] = str(item.pop('_id'))
+            if skip_geometry:
+                item['geometry'] = None
 
         return featurelist, matchCount
 
-    def query(self, startindex=0, limit=10, resulttype='results',
+    def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
-              select_properties=[], skip_geometry=False, q=None):
+              select_properties=[], skip_geometry=False, q=None, **kwargs):
         """
         query the provider
 
@@ -126,10 +139,9 @@ class MongoProvider(BaseProvider):
                       ASCENDING if (sort['order'] == '+') else DESCENDING)
                      for sort in sortby]
 
-        featurelist, matchcount = self._get_feature_list(filterobj,
-                                                         sortList=sort_list,
-                                                         skip=startindex,
-                                                         maxitems=limit)
+        featurelist, matchcount = self._get_feature_list(
+            filterobj, sortList=sort_list, skip=offset, maxitems=limit,
+            skip_geometry=skip_geometry)
 
         if resulttype == 'hits':
             featurelist = []
@@ -143,7 +155,7 @@ class MongoProvider(BaseProvider):
 
         return feature_collection
 
-    def get(self, identifier):
+    def get(self, identifier, **kwargs):
         """
         query the provider by id
 
@@ -155,7 +167,7 @@ class MongoProvider(BaseProvider):
         if featurelist:
             return featurelist[0]
         else:
-            err = 'item {} not found'.format(identifier)
+            err = f'item {identifier} not found'
             LOGGER.error(err)
             raise ProviderItemNotFoundError(err)
 
