@@ -5,7 +5,7 @@
 #
 # Copyright (c) 2019 Just van den Broecke
 # Copyright (c) 2020 Francesco Bartoli
-# Copyright (c) 2021 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -85,7 +85,7 @@ class OGRProvider(BaseProvider):
             name: OGR
             data:
                 source_type: WFS
-                source: WFS:http://geodata.nationaalgeoregister.nl/rdinfo/wfs?
+                source: WFS:https://service.pdok.nl/kadaster/rdinfo/wfs/v1_0?
                 source_srs: EPSG:28992
                 target_srs: EPSG:4326
                 source_capabilities:
@@ -172,7 +172,7 @@ class OGRProvider(BaseProvider):
         self._load_source_helper(self.data_def['source_type'])
 
         # Layer name is required
-        self.layer_name = provider_def.get('layer', None)
+        self.layer_name = provider_def.get('layer')
         if not self.layer_name:
             msg = 'Need explicit \'layer\' attr in provider config'
             LOGGER.error(msg)
@@ -193,7 +193,7 @@ class OGRProvider(BaseProvider):
         source_type = self.data_def['source_type']
         self.driver = self.ogr.GetDriverByName(source_type)
         if not self.driver:
-            msg = 'No Driver for Source: {}'.format(source_type)
+            msg = f'No Driver for Source: {source_type}'
             LOGGER.error(msg)
             raise Exception(msg)
         if self.open_options:
@@ -206,8 +206,7 @@ class OGRProvider(BaseProvider):
                 LOGGER.error(err)
                 raise ProviderConnectionError(err)
             except Exception:
-                msg = 'Ignore errors during the connection for Driver \
-                    {}'.format(source_type)
+                msg = f'Ignore errors during the connection for Driver {source_type}'  # noqa
                 LOGGER.error(msg)
                 self.conn = _ignore_gdal_error(
                     self.gdal, 'OpenEx', self.data_def['source'],
@@ -220,8 +219,7 @@ class OGRProvider(BaseProvider):
                 LOGGER.error(err)
                 raise ProviderConnectionError(err)
             except Exception:
-                msg = 'Ignore errors during the connection for Driver \
-                    {}'.format(source_type)
+                msg = f'Ignore errors during the connection for Driver {source_type}'  # noqa
                 LOGGER.error(msg)
                 # ignore errors for ESRIJSON not having geometry member
                 # see https://github.com/OSGeo/gdal/commit/38b0feed67f80ded32be6c508323d862e1a14474 # noqa
@@ -275,6 +273,12 @@ class OGRProvider(BaseProvider):
 
                 fields[fieldName] = {'type': fieldName2}
 
+                if fieldName2 == 'datetime':
+                    fields[fieldName] = {
+                        'type': 'string',
+                        'format': 'date-time'
+                    }
+
                 # fieldWidth = layer_defn.GetFieldDefn(fld).GetWidth()
                 # GetPrecision = layer_defn.GetFieldDefn(fld).GetPrecision()
 
@@ -289,13 +293,13 @@ class OGRProvider(BaseProvider):
 
         return fields
 
-    def query(self, startindex=0, limit=10, resulttype='results',
+    def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
               select_properties=[], skip_geometry=False, q=None, **kwargs):
         """
         Query OGR source
 
-        :param startindex: starting record to return (default 0)
+        :param offset: starting record to return (default 0)
         :param limit: number of records to return (default 10)
         :param resulttype: return results or hit limit (default results)
         :param bbox: bounding box [minx,miny,maxx,maxy]
@@ -311,18 +315,16 @@ class OGRProvider(BaseProvider):
         result = None
         try:
             if self.source_capabilities['paging']:
-                self.source_helper.enable_paging(startindex, limit)
+                self.source_helper.enable_paging(offset, limit)
 
             layer = self._get_layer()
 
             if bbox:
                 LOGGER.debug('processing bbox parameter')
-                minx, miny, maxx, maxy = bbox
+                minx, miny, maxx, maxy = [float(b) for b in bbox]
 
-                wkt = "POLYGON (({minx} {miny},{minx} {maxy},{maxx} {maxy}," \
-                      "{maxx} {miny},{minx} {miny}))".format(
-                        minx=float(minx), miny=float(miny),
-                        maxx=float(maxx), maxy=float(maxy))
+                wkt = f"POLYGON (({minx} {miny},{minx} {maxy},{maxx} {maxy}," \
+                      f"{maxx} {miny},{minx} {miny}))"
 
                 polygon = self.ogr.CreateGeometryFromWkt(wkt)
                 if self.transform_in:
@@ -337,10 +339,7 @@ class OGRProvider(BaseProvider):
                 LOGGER.debug('processing properties')
 
                 attribute_filter = ' and '.join(
-                    map(
-                        lambda x: '{} = \'{}\''.format(x[0], x[1]),
-                        properties
-                    )
+                    map(lambda x: f'{x[0]} = \'{x[1]}\'', properties)
                 )
 
                 LOGGER.debug(attribute_filter)
@@ -353,7 +352,8 @@ class OGRProvider(BaseProvider):
                 result = self._response_feature_hits(layer)
             elif resulttype == 'results':
                 LOGGER.debug('results specified')
-                result = self._response_feature_collection(layer, limit)
+                result = self._response_feature_collection(
+                    layer, limit, skip_geometry=skip_geometry)
             else:
                 LOGGER.error('Invalid resulttype: %s' % resulttype)
 
@@ -382,11 +382,10 @@ class OGRProvider(BaseProvider):
         """
         result = None
         try:
-            LOGGER.debug('Fetching identifier {}'.format(identifier))
+            LOGGER.debug(f'Fetching identifier {identifier}')
             layer = self._get_layer()
 
-            layer.SetAttributeFilter("{field} = '{id}'".format(
-                field=self.id_field, id=identifier))
+            layer.SetAttributeFilter(f"{self.id_field} = '{identifier}'")
 
             ogr_feature = self._get_next_feature(layer, identifier)
             result = self._ogr_feature_to_json(ogr_feature)
@@ -410,7 +409,7 @@ class OGRProvider(BaseProvider):
         return result
 
     def __repr__(self):
-        return '<OGRProvider> {}'.format(self.data)
+        return f'<OGRProvider> {self.data}'
 
     def _load_source_helper(self, source_type):
         """
@@ -435,9 +434,9 @@ class OGRProvider(BaseProvider):
     def _get_next_feature(self, layer, feature_id):
         try:
             if layer.GetFeatureCount() == 0:
-                LOGGER.error("item {} is not found".format(feature_id))
-                raise ProviderItemNotFoundError(
-                    "item {} not found".format(feature_id))
+                msg = f"item {feature_id} is not found"
+                LOGGER.error(msg)
+                raise ProviderItemNotFoundError(msg)
             # Ignore gdal error
             next_feature = _ignore_gdal_error(layer, 'GetNextFeature')
             if next_feature:
@@ -448,29 +447,31 @@ class OGRProvider(BaseProvider):
                     )
             else:
                 raise RuntimeError(
-                    "GDAL has returned a null feature for item {}".format(
-                        feature_id))
+                    f"GDAL has returned a null feature for item {feature_id}")
             return next_feature
         except RuntimeError as gdalerr:
             LOGGER.error(self.gdal.GetLastErrorMsg())
             raise gdalerr
 
-    def _ogr_feature_to_json(self, ogr_feature):
+    def _ogr_feature_to_json(self, ogr_feature, skip_geometry=False):
         geom = ogr_feature.GetGeometryRef()
         if self.transform_out:
             # Optionally reproject the geometry
             geom.Transform(self.transform_out)
 
         json_feature = ogr_feature.ExportToJson(as_object=True)
+        if skip_geometry:
+            json_feature['geometry'] = None
         try:
-            json_feature['id'] = json_feature['properties'].pop(self.id_field)
-        except Exception as err:
-            LOGGER.error(err)
-            json_feature['id'] = ogr_feature.GetFID()
+            json_feature['id'] = json_feature['properties'].pop(
+                self.id_field, json_feature['id']
+            )
+        except KeyError as err:
+            LOGGER.error(f"Cannot use configured id_field nor fid as id, err={err}")  # noqa
 
         return json_feature
 
-    def _response_feature_collection(self, layer, limit):
+    def _response_feature_collection(self, layer, limit, skip_geometry=False):
         """
         Assembles output from Layer query as
         GeoJSON FeatureCollection structure.
@@ -492,7 +493,8 @@ class OGRProvider(BaseProvider):
             ogr_feature = _ignore_gdal_error(layer, 'GetNextFeature')
             count = 0
             while ogr_feature is not None:
-                json_feature = self._ogr_feature_to_json(ogr_feature)
+                json_feature = self._ogr_feature_to_json(
+                    ogr_feature, skip_geometry=skip_geometry)
 
                 feature_collection['features'].append(json_feature)
 
@@ -563,14 +565,13 @@ class SourceHelper:
         layer = self.provider.conn.GetLayerByName(self.provider.layer_name)
 
         if not layer:
-            msg = 'Cannot get Layer {} from OGR Source'.\
-                format(self.provider.layer_name)
+            msg = f'Cannot get Layer {self.provider.layer_name} from OGR Source'  # noqa
             LOGGER.error(msg)
             raise Exception(msg)
 
         return layer
 
-    def enable_paging(self, startindex=-1, limit=-1):
+    def enable_paging(self, offset=-1, limit=-1):
         """
         Enable paged access to dataset (OGR Driver-specific)
 
@@ -602,7 +603,7 @@ class CommonSourceHelper(SourceHelper):
 
         super().__init__(provider)
 
-        self.startindex = -1
+        self.offset = -1
         self.limit = -1
         self.result_set = None
 
@@ -620,20 +621,19 @@ class CommonSourceHelper(SourceHelper):
         try:
             self.provider.conn.ReleaseResultSet(self.result_set)
         except Exception as err:
-            msg = 'ReleaseResultSet exception for Layer {}'.format(
-                self.provider.layer_name)
+            msg = f'ReleaseResultSet exception for Layer {self.provider.layer_name}'  # noqa
             LOGGER.error(msg, err)
         finally:
             self.result_set = None
 
-    def enable_paging(self, startindex=-1, limit=-1):
+    def enable_paging(self, offset=-1, limit=-1):
         """
         Enable paged access to dataset (OGR Driver-specific)
         using OGR SQL https://gdal.org/user/ogr_sql_dialect.html
         e.g. SELECT * FROM poly LIMIT 10 OFFSET 30
 
         """
-        self.startindex = startindex
+        self.offset = offset
         self.limit = limit
 
     def disable_paging(self):
@@ -646,29 +646,25 @@ class CommonSourceHelper(SourceHelper):
     def get_layer(self):
         """
         Gets OGR Layer from opened OGR dataset.
-        When startindex defined 1 or greater will invoke
+        When offset defined 1 or greater will invoke
         OGR SQL SELECT with LIMIT and OFFSET and return
         as Layer as ResultSet from ExecuteSQL on dataset.
         :return: OGR layer object
         """
-        if self.startindex <= 0:
+        if self.offset <= 0:
             return SourceHelper.get_layer(self)
 
         self.close()
 
-        sql = 'SELECT * FROM "{ds_name}" LIMIT {limit} OFFSET {offset}'.format(
-            ds_name=self.provider.layer_name,
-            limit=self.limit,
-            offset=self.startindex)
+        sql = f'SELECT * FROM "{self.provider.layer_name}" LIMIT {self.limit} OFFSET {self.offset}'  # noqa
         self.result_set = self.provider.conn.ExecuteSQL(sql)
 
         # Reset since needs to be set each time explicitly
-        self.startindex = -1
+        self.offset = -1
         self.limit = -1
 
         if not self.result_set:
-            msg = 'Cannot get Layer {} via ExecuteSQL'.format(
-                self.provider.layer_name)
+            msg = f'Cannot get Layer {self.provider.layer_name} via ExecuteSQL'
             LOGGER.error(msg)
             raise Exception(msg)
 
@@ -688,16 +684,16 @@ class ESRIJSONHelper(CommonSourceHelper):
 
         super().__init__(provider)
 
-    def enable_paging(self, startindex=-1, limit=-1):
+    def enable_paging(self, offset=-1, limit=-1):
         """
         Enable paged access to dataset (OGR Driver-specific)
 
         """
-        if startindex < 0:
+        if offset < 0:
             return
 
         self.provider.open_options.update(FEATURE_SERVER_PAGING=True)
-        self.startindex = startindex
+        self.offset = offset
         self.limit = limit
 
     def disable_paging(self):
@@ -710,29 +706,25 @@ class ESRIJSONHelper(CommonSourceHelper):
     def get_layer(self):
         """
         Gets OGR Layer from opened OGR dataset.
-        When startindex defined 1 or greater will invoke
+        When offset defined 1 or greater will invoke
         OGR SQL SELECT with LIMIT and OFFSET and return
         as Layer as ResultSet from ExecuteSQL on dataset.
         :return: OGR layer object
         """
-        if self.startindex <= 0:
+        if self.offset <= 0:
             return CommonSourceHelper.get_layer(self)
 
         self.close()
 
-        sql = "SELECT * FROM {ds_name} LIMIT {limit} OFFSET {offset}".format(
-            ds_name=self.provider.layer_name,
-            limit=self.limit,
-            offset=self.startindex)
+        sql = f"SELECT * FROM {self.provider.layer_name} LIMIT {self.limit} OFFSET {self.offset}"  # noqa
         self.result_set = self.provider.conn.ExecuteSQL(sql)
 
         # Reset since needs to be set each time explicitly
-        self.startindex = -1
+        self.offset = -1
         self.limit = -1
 
         if not self.result_set:
-            msg = 'Cannot get Layer {} via ExecuteSQL'.format(
-                self.provider.layer_name)
+            msg = f'Cannot get Layer {self.provider.layer_name} via ExecuteSQL'
             LOGGER.error(msg)
             raise Exception(msg)
 
@@ -752,19 +744,19 @@ class WFSHelper(SourceHelper):
 
         super().__init__(provider)
 
-    def enable_paging(self, startindex=-1, limit=-1):
+    def enable_paging(self, offset=-1, limit=-1):
         """
         Enable paged access to dataset (OGR Driver-specific)
 
         """
 
-        if startindex < 0:
+        if offset < 0:
             return
 
         self.provider.gdal.SetConfigOption(
             'OGR_WFS_PAGING_ALLOWED', 'ON')
         self.provider.gdal.SetConfigOption(
-            'OGR_WFS_BASE_START_INDEX', str(startindex))
+            'OGR_WFS_BASE_START_INDEX', str(offset))
         self.provider.gdal.SetConfigOption(
             'OGR_WFS_PAGE_SIZE', str(limit))
 
