@@ -1,8 +1,10 @@
 # =================================================================
 #
 # Authors: Benjamin Webb <benjamin.miller.webb@gmail.com>
+# Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2021 Benjamin Webb
+# Copyright (c) 2023 Benjamin Webb
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -27,13 +29,13 @@
 #
 # =================================================================
 
-from requests import get, codes
-from requests.compat import urljoin
+from requests import Session, get, codes
 import logging
 from pygeoapi.provider.base import (BaseProvider, ProviderQueryError,
                                     ProviderConnectionError,
                                     ProviderItemNotFoundError)
-from pygeoapi.util import yaml_load
+from json.decoder import JSONDecodeError
+from pygeoapi.util import yaml_load, url_join
 
 LOGGER = logging.getLogger(__name__)
 
@@ -46,26 +48,18 @@ ENTITY = {
     }
 _EXPAND = {
     'Things': """
-        Locations,
-        Datastreams(
-            $select=@iot.id,properties
-            )
+        Locations
+        ,Datastreams
     """,
     'Observations': """
-        Datastream(
-            $select=@iot.id,properties
-            ),
-        FeatureOfInterest
+        Datastream
+        ,FeatureOfInterest
     """,
     'Datastreams': """
         Sensor
         ,ObservedProperty
-        ,Thing(
-            $select=@iot.id,properties
-            )
-        ,Thing/Locations(
-            $select=location
-            )
+        ,Thing
+        ,Thing/Locations
         ,Observations(
             $select=@iot.id;
             $orderby=phenomenonTime_desc
@@ -97,7 +91,7 @@ class SensorThingsProvider(BaseProvider):
         super().__init__(provider_def)
         try:
             self.entity = provider_def['entity']
-            self._url = ''.join((self.data, self.entity))
+            self._url = url_join(self.data, self.entity)
         except KeyError:
             raise RuntimeError('name/type/data are required')
 
@@ -124,7 +118,15 @@ class SensorThingsProvider(BaseProvider):
         if not self.fields:
             p = {'$expand': EXPAND[self.entity], '$top': 1}
             r = get(self._url, params=p)
-            results = r.json()['value'][0]
+            try:
+                results = r.json()['value'][0]
+            except JSONDecodeError as err:
+                LOGGER.error(f'Entity {self.entity} error: {err}')
+                LOGGER.error(f'Bad url response at {r.url}')
+                raise ProviderQueryError(err)
+            except IndexError:
+                LOGGER.warning('could not get fields; returning empty set')
+                return {}
 
             for (n, v) in results.items():
                 if isinstance(v, (int, float)) or \
@@ -135,13 +137,13 @@ class SensorThingsProvider(BaseProvider):
 
         return self.fields
 
-    def query(self, startindex=0, limit=10, resulttype='results',
+    def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
               select_properties=[], skip_geometry=False, q=None, **kwargs):
         """
         STA query
 
-        :param startindex: starting record to return (default 0)
+        :param offset: starting record to return (default 0)
         :param limit: number of records to return (default 10)
         :param resulttype: return results or hit limit (default results)
         :param bbox: bounding box [minx,miny,maxx,maxy]
@@ -155,7 +157,7 @@ class SensorThingsProvider(BaseProvider):
         :returns: dict of GeoJSON FeatureCollection
         """
 
-        return self._load(startindex, limit, resulttype, bbox=bbox,
+        return self._load(offset, limit, resulttype, bbox=bbox,
                           datetime_=datetime_, properties=properties,
                           sortby=sortby, select_properties=select_properties,
                           skip_geometry=skip_geometry)
@@ -192,10 +194,11 @@ class SensorThingsProvider(BaseProvider):
                 for p in rs['providers']:
                     # Validate linkable provider
                     if (p['name'] != 'SensorThings'
-                            or not p.get('intralink', False)):
+                            or not p.get('intralink', False)
+                            or p['data'] != self.data):
                         continue
 
-                    if p.get('default', False) is True:
+                    if p.get('default', False):
                         _entity = p['entity']
                         uri = p['uri_field']
 
@@ -204,13 +207,13 @@ class SensorThingsProvider(BaseProvider):
                         'n': name, 'u': uri
                     })
 
-    def _load(self, startindex=0, limit=10, resulttype='results',
+    def _load(self, offset=0, limit=10, resulttype='results',
               identifier=None, bbox=[], datetime_=None, properties=[],
               sortby=[], select_properties=[], skip_geometry=False, q=None):
         """
         Private function: Load STA data
 
-        :param startindex: starting record to return (default 0)
+        :param offset: starting record to return (default 0)
         :param limit: number of records to return (default 10)
         :param resulttype: return results or hit limit (default results)
         :param bbox: bounding box [minx,miny,maxx,maxy]
@@ -229,7 +232,7 @@ class SensorThingsProvider(BaseProvider):
         # Make params
         params = {
             '$expand': EXPAND[self.entity],
-            '$skip': str(startindex),
+            '$skip': str(offset),
             '$top': str(limit),
             '$count': 'true'
         }
@@ -238,34 +241,44 @@ class SensorThingsProvider(BaseProvider):
         if sortby:
             params['$orderby'] = self._make_orderby(sortby)
 
+        # Start session
+        s = Session()
+
         # Form URL for GET request
         LOGGER.debug('Sending query')
         if identifier:
-            r = get(f'{self._url}({identifier})', params=params)
+            r = s.get(f'{self._url}({identifier})', params=params)
         else:
-            r = get(self._url, params=params)
+            r = s.get(self._url, params=params)
 
         if r.status_code == codes.bad:
             LOGGER.error('Bad http response code')
             raise ProviderConnectionError('Bad http response code')
-
         response = r.json()
+
         # if hits, return count
         if resulttype == 'hits':
             LOGGER.debug('Returning hits')
             feature_collection['numberMatched'] = response.get('@iot.count')
             return feature_collection
 
+        # Query if values are less than expected
         v = [response, ] if identifier else response.get('value')
-        # if values are less than expected, query for more
         hits_ = 1 if identifier else min(limit, response.get('@iot.count'))
         while len(v) < hits_:
             LOGGER.debug('Fetching next set of values')
-            r = get(response.get('@iot.nextLink'), params={'$skip': len(v)})
-            response = r.json()
-            v.extend(response.get('value'))
+            next_ = response.get('@iot.nextLink')
+            if next_ is None:
+                break
+            else:
+                with s.get(next_) as r:
+                    response = r.json()
+                    v.extend(response.get('value'))
 
-        # properties filter & display
+        # End session
+        s.close()
+
+        # Properties filter & display
         keys = (() if not self.properties and not select_properties else
                 set(self.properties) | set(select_properties))
 
@@ -400,10 +413,13 @@ class SensorThingsProvider(BaseProvider):
         if self.entity == 'Things':
             extra_props = entity['Locations'][0].get('properties', {})
             entity['properties'].update(extra_props)
+        elif 'Thing' in entity.keys():
+            t = entity.get('Thing')
+            extra_props = t['Locations'][0].get('properties', {})
+            t['properties'].update(extra_props)
 
         for k, v in entity.items():
             # Create intra links
-            path_ = 'collections/{}/items/{}'
             ks = f'{k}s'
             if self.uri_field is not None and k in ['properties']:
                 uri = v.get(self.uri_field, '')
@@ -414,14 +430,9 @@ class SensorThingsProvider(BaseProvider):
                         v[i] = _v['properties'][self._linkables[k]['u']]
                     continue
                 for i, _v in enumerate(v):
-                    id = _v[self.id_field]
-                    id = f"'{id}'" if isinstance(id, str) else str(id)
-                    v[i] = urljoin(
-                        self._rel_link,
-                        path_.format(
-                            self._linkables[k]['n'], id
-                        )
-                    )
+                    id_ = _v[self.id_field]
+                    id_ = f"'{id_}'" if isinstance(id_, str) else str(id_)
+                    v[i] = url_join(self._rel_link, f"collections/{self._linkables[k]['n']}/items/{id_}")  # noqa
 
             elif ks in self._linkables.keys():
                 if self._linkables[ks]['u'] != '':
@@ -429,15 +440,12 @@ class SensorThingsProvider(BaseProvider):
                     continue
                 id = v[self.id_field]
                 id = f"'{id}'" if isinstance(id, str) else str(id)
-                entity[k] = urljoin(
+                entity[k] = url_join(
                     self._rel_link,
-                    path_.format(
-                        self._linkables[ks]['n'], id
-                    )
-                )
+                    f"collections/{self._linkables[ks]['n']}/items/{id_}")
 
         # Make properties block
-        if entity.get('properties', None):
+        if entity.get('properties'):
             entity.update(entity.pop('properties'))
 
         if keys:
@@ -453,4 +461,4 @@ class SensorThingsProvider(BaseProvider):
         return entity
 
     def __repr__(self):
-        return '<SensorThingsProvider> {}, {}'.format(self.data, self.entity)
+        return f'<SensorThingsProvider> {self.data}, {self.entity}'
