@@ -5,7 +5,7 @@
 #          Francesco Bartoli <xbartolone@gmail.com>
 #
 # Copyright (c) 2018 Jorge Samuel Mendes de Jesus
-# Copyright (c) 2021 Tom Kralidis
+# Copyright (c) 2023 Tom Kralidis
 # Copyright (c) 2020 Francesco Bartoli
 #
 # Permission is hereby granted, free of charge, to any person
@@ -68,10 +68,10 @@ class SQLiteGPKGProvider(BaseProvider):
         self.geom_col = None
 
         LOGGER.debug('Setting SQLite properties:')
-        LOGGER.debug('Data source: {}'.format(self.data))
-        LOGGER.debug('Name: {}'.format(self.name))
-        LOGGER.debug('ID_field: {}'.format(self.id_field))
-        LOGGER.debug('Table: {}'.format(self.table))
+        LOGGER.debug(f'Data source: {self.data}')
+        LOGGER.debug(f'Name: {self.name}')
+        LOGGER.debug(f'ID_field: {self.id_field}')
+        LOGGER.debug(f'Table: {self.table}')
 
         self.cursor = self.__load()
 
@@ -89,9 +89,17 @@ class SQLiteGPKGProvider(BaseProvider):
 
         if not self.fields:
             results = self.cursor.execute(
-                'PRAGMA table_info({})'.format(self.table)).fetchall()
+                f'PRAGMA table_info({self.table})').fetchall()
             for item in results:
-                self.fields[item['name']] = {'type': item['type']}
+                json_type = None
+
+                if item['type'] in ['INTEGER', 'REAL']:
+                    json_type = 'number'
+                elif item['type'].startswith('TEXT') or item['type'] == 'BLOB':
+                    json_type = 'string'
+
+                if json_type is not None:
+                    self.fields[item['name']] = {'type': json_type}
 
         return self.fields
 
@@ -116,23 +124,23 @@ class SQLiteGPKGProvider(BaseProvider):
 
         if properties:
             where_clause += " AND ".join(
-                ["{}=?".format(k) for k, v in properties])
+                [f"{k}=?" for k, v in properties])
             where_values += where_values + tuple((v for k, v in properties))
 
         if bbox:
             if properties:
                 where_clause += " AND "
-            where_clause += " Intersects({}, \
-                BuildMbr(?,?,?,?)) ".format(self.geom_col)
+            where_clause += f" Intersects({self.geom_col}, BuildMbr(?,?,?,?)) "
             where_values += tuple(bbox)
         # WHERE continent=? <class 'tuple'>: ('Europe',)
         return where_clause, where_values
 
-    def __response_feature(self, row_data):
+    def __response_feature(self, row_data, skip_geometry=False):
         """
         Assembles GeoJSON output from DB query
 
         :param row_data: DB row result
+        :param skip_geometry: whether to skip geometry (default False)
 
         :returns: `dict` of GeoJSON Feature
         """
@@ -143,8 +151,11 @@ class SQLiteGPKGProvider(BaseProvider):
                 'type': 'Feature'
             }
             feature["geometry"] = json.loads(
-                rd.pop('AsGeoJSON({})'.format(self.geom_col))
+                rd.pop(f'AsGeoJSON({self.geom_col})')
                 )
+            if skip_geometry:
+                feature["geometry"] = None
+
             feature['properties'] = rd
             feature['id'] = feature['properties'].pop(self.id_field)
 
@@ -181,7 +192,7 @@ class SQLiteGPKGProvider(BaseProvider):
         try:
             conn.enable_load_extension(True)
         except AttributeError as err:
-            LOGGER.error('Extension loading not enabled: {}'.format(err))
+            LOGGER.error(f'Extension loading not enabled: {err}')
             raise ProviderConnectionError()
 
         conn.row_factory = sqlite3.Row
@@ -189,10 +200,9 @@ class SQLiteGPKGProvider(BaseProvider):
         # conn.set_trace_callback(LOGGER.debug)
         cursor = conn.cursor()
         try:
-            cursor.execute("SELECT load_extension('{}')".format(
-                SPATIALITE_EXTENSION))
+            cursor.execute(f"SELECT load_extension('{SPATIALITE_EXTENSION}')")
         except sqlite3.OperationalError as err:
-            LOGGER.error('Extension loading error: {}'.format(err))
+            LOGGER.error(f'Extension loading error: {err}')
             raise ProviderConnectionError()
         result = cursor.fetchall()
 
@@ -210,6 +220,9 @@ class SQLiteGPKGProvider(BaseProvider):
             self.application_id = 0
 
         if self.application_id:
+            geometry_columns_table = 'gpkg_geometry_columns'
+            geometry_columns_table_name = 'table_name'
+            geometry_columns_column_name = 'column_name'
             cursor.execute("SELECT AutoGPKGStart()")
             result = cursor.fetchall()
             if result[0][0] >= 1:
@@ -217,48 +230,57 @@ class SQLiteGPKGProvider(BaseProvider):
             else:
                 LOGGER.info("SELECT AutoGPKGStart() returned 0." +
                             "Detected GPKG but couldn't load support")
-                raise InvalidPluginError
-
-        if self.application_id:
-            self.geom_col = "geom"
+                raise InvalidPluginError()
         else:
-            self.geom_col = "geometry"
+            geometry_columns_table = 'geometry_columns'
+            geometry_columns_column_name = 'f_geometry_column'
+            geometry_columns_table_name = 'f_table_name'
 
         try:
-            cursor.execute('PRAGMA table_info({})'.format(self.table))
+            cursor.execute(f'PRAGMA table_info({self.table})')
             result = cursor.fetchall()
         except sqlite3.OperationalError:
-            LOGGER.error('Couldnt find table: {}'.format(self.table))
+            LOGGER.error(f'Could not find table: {self.table}')
             raise ProviderConnectionError()
 
+        LOGGER.debug('Determining name of geometry column')
+        cursor.execute(f"SELECT {geometry_columns_column_name} FROM {geometry_columns_table} WHERE {geometry_columns_table_name} = '{self.table}'")  # noqa
+        geometry_column = cursor.fetchall()
+
+        if geometry_column:
+            LOGGER.debug("Found geometry column")
+            self.geom_col = geometry_column[0][0]
+        else:
+            msg = 'No geometry column found'
+            LOGGER.error(msg)
+            raise ProviderConnectionError(msg)
+
         try:
             assert len(result), 'Table not found'
             assert len([item for item in result
                         if self.id_field in item]), 'id_field not present'
-
         except AssertionError:
-            raise InvalidPluginError
+            raise InvalidPluginError()
 
         self.columns = [item[1] for item in result if item[1]
                         not in [self.geom_col, self.geom_col.upper()]]
-        self.columns = ','.join(self.columns)+',AsGeoJSON({})'.format(
-            self.geom_col)
+        self.columns = ','.join(self.columns)+f',AsGeoJSON({self.geom_col})'
 
         if self.application_id:
-            self.table = "vgpkg_{}".format(self.table)
+            self.table = f"vgpkg_{self.table}"
 
         return cursor
 
-    def query(self, startindex=0, limit=10, resulttype='results',
+    def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
               select_properties=[], skip_geometry=False, q=None, **kwargs):
         """
         Query SQLite/GPKG for all the content.
         e,g: http://localhost:5000/collections/countries/items?
-        limit=5&startindex=2&resulttype=results&continent=Europe&admin=Albania&bbox=29.3373,-3.4099,29.3761,-3.3924
+        limit=5&offset=2&resulttype=results&continent=Europe&admin=Albania&bbox=29.3373,-3.4099,29.3761,-3.3924
         http://localhost:5000/collections/countries/items?continent=Africa&bbox=29.3373,-3.4099,29.3761,-3.3924
 
-        :param startindex: starting record to return (default 0)
+        :param offset: starting record to return (default 0)
         :param limit: number of records to return (default 10)
         :param resulttype: return results or hit limit (default results)
         :param bbox: bounding box [minx,miny,maxx,maxy]
@@ -278,26 +300,24 @@ class SQLiteGPKGProvider(BaseProvider):
 
         if resulttype == 'hits':
 
-            sql_query = "SELECT COUNT(*) as hits FROM {} {} ".format(
-                self.table, where_clause)
+            sql_query = f"SELECT COUNT(*) as hits FROM {self.table} {where_clause} "  # noqa
 
             res = self.cursor.execute(sql_query, where_values)
 
             hits = res.fetchone()["hits"]
             return self.__response_feature_hits(hits)
 
-        sql_query = "SELECT DISTINCT {} from \
-            {} {} limit ? offset ?".format(
-                self.columns, self.table, where_clause)
+        sql_query = f"SELECT DISTINCT {self.columns} from \
+            {self.table} {where_clause} limit ? offset ?"
 
-        end_index = startindex + limit
+        end_index = offset + limit
 
-        LOGGER.debug('SQL Query: {}'.format(sql_query))
-        LOGGER.debug('Start Index: {}'.format(startindex))
-        LOGGER.debug('End Index: {}'.format(end_index))
+        LOGGER.debug(f'SQL Query: {sql_query}')
+        LOGGER.debug(f'Start Index: {offset}')
+        LOGGER.debug(f'End Index: {end_index}')
 
         row_data = self.cursor.execute(
-            sql_query, where_values + (limit, startindex))
+            sql_query, where_values + (limit, offset))
 
         feature_collection = {
             'type': 'FeatureCollection',
@@ -306,7 +326,7 @@ class SQLiteGPKGProvider(BaseProvider):
 
         for rd in row_data:
             feature_collection['features'].append(
-                self.__response_feature(rd))
+                self.__response_feature(rd, skip_geometry=skip_geometry))
 
         return feature_collection
 
@@ -322,12 +342,11 @@ class SQLiteGPKGProvider(BaseProvider):
 
         LOGGER.debug('Get item from SQLite/GPKG')
 
-        sql_query = 'SELECT {} FROM \
-            {} WHERE {}==?;'.format(
-                self.columns, self.table, self.id_field)
+        sql_query = f'SELECT {self.columns} FROM \
+            {self.table} WHERE {self.id_field}==?;'
 
-        LOGGER.debug('SQL Query: {}'.format(sql_query))
-        LOGGER.debug('Identifier: {}'.format(identifier))
+        LOGGER.debug(f'SQL Query: {sql_query}')
+        LOGGER.debug(f'Identifier: {identifier}')
 
         row_data = self.cursor.execute(sql_query, (identifier, )).fetchone()
 
@@ -335,9 +354,9 @@ class SQLiteGPKGProvider(BaseProvider):
         if feature:
             return feature
         else:
-            err = 'item {} not found'.format(identifier)
+            err = f'item {identifier} not found'
             LOGGER.error(err)
             raise ProviderItemNotFoundError(err)
 
     def __repr__(self):
-        return '<SQLiteGPKGProvider> {}, {}'.format(self.data, self.table)
+        return f'<SQLiteGPKGProvider> {self.data}, {self.table}'
