@@ -1,8 +1,10 @@
 # =================================================================
 #
 # Authors: Gregory Petrochenkov <gpetrochenkov@usgs.gov>
+# Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
 # Copyright (c) 2020 Gregory Petrochenkov
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -153,7 +155,7 @@ class XarrayProvider(BaseProvider):
         }
 
         for name, var in self._data.variables.items():
-            LOGGER.debug('Determining rangetype for {}'.format(name))
+            LOGGER.debug(f'Determining rangetype for {name}')
 
             desc, units = None, None
             if len(var.shape) >= 3:
@@ -167,12 +169,11 @@ class XarrayProvider(BaseProvider):
                     'type': 'Quantity',
                     'name': var.attrs.get('long_name') or desc,
                     'encodingInfo': {
-                        'dataType': 'http://www.opengis.net/def/dataType/OGC/0/{}'.format(str(var.dtype))  # noqa
+                        'dataType': f'http://www.opengis.net/def/dataType/OGC/0/{var.dtype}'  # noqa
                     },
                     'nodata': 'null',
                     'uom': {
-                        'id': 'http://www.opengis.net/def/uom/UCUM/{}'.format(
-                             units),
+                        'id': f'http://www.opengis.net/def/uom/UCUM/{units}',
                         'type': 'UnitReference',
                         'code': units
                     },
@@ -183,31 +184,32 @@ class XarrayProvider(BaseProvider):
 
         return rangetype
 
-    def query(self, range_subset=[], subsets={}, bbox=[], datetime_=None,
-              format_='json', **kwargs):
+    def query(self, properties=[], subsets={}, bbox=[], bbox_crs=4326,
+              datetime_=None, format_='json', **kwargs):
         """
          Extract data from collection collection
 
-        :param range_subset: list of data variables to return (all if blank)
+        :param properties: list of data variables to return (all if blank)
         :param subsets: dict of subset names with lists of ranges
         :param bbox: bounding box [minx,miny,maxx,maxy]
+        :param bbox_crs: CRS of bounding box
         :param datetime_: temporal (datestamp or extent)
         :param format_: data format of output
 
         :returns: coverage data as dict of CoverageJSON or native format
         """
 
-        if not range_subset and not subsets and format_ != 'json':
+        if not properties and not subsets and format_ != 'json':
             LOGGER.debug('No parameters specified, returning native data')
             if format_ == 'zarr':
                 return _get_zarr_data(self._data)
             else:
                 return read_data(self.data)
 
-        if len(range_subset) < 1:
-            range_subset = self.fields
+        if len(properties) < 1:
+            properties = self.fields
 
-        data = self._data[[*range_subset]]
+        data = self._data[[*properties]]
 
         if any([self._coverage_properties['x_axis_label'] in subsets,
                 self._coverage_properties['y_axis_label'] in subsets,
@@ -218,7 +220,7 @@ class XarrayProvider(BaseProvider):
 
             query_params = {}
             for key, val in subsets.items():
-                LOGGER.debug('Processing subset: {}'.format(key))
+                LOGGER.debug(f'Processing subset: {key}')
                 if data.coords[key].values[0] > data.coords[key].values[-1]:
                     LOGGER.debug('Reversing slicing from high to low')
                     query_params[key] = slice(val[1], val[0])
@@ -238,6 +240,8 @@ class XarrayProvider(BaseProvider):
                     query_params[self._coverage_properties['y_axis_label']] = \
                         slice(bbox[1], bbox[3])
 
+                LOGGER.debug('bbox_crs is not currently handled')
+
             if datetime_ is not None:
                 if self._coverage_properties['time_axis_label'] in subsets:
                     msg = 'datetime and temporal subsetting are exclusive'
@@ -254,7 +258,7 @@ class XarrayProvider(BaseProvider):
                     else:
                         query_params[self.time_field] = datetime_
 
-            LOGGER.debug('Query parameters: {}'.format(query_params))
+            LOGGER.debug(f'Query parameters: {query_params}')
             try:
                 data = data.sel(query_params)
             except Exception as err:
@@ -290,7 +294,7 @@ class XarrayProvider(BaseProvider):
         LOGGER.debug('Serializing data in memory')
         if format_ == 'json':
             LOGGER.debug('Creating output in CoverageJSON')
-            return self.gen_covjson(out_meta, data, range_subset)
+            return self.gen_covjson(out_meta, data, properties)
         elif format_ == 'zarr':
             LOGGER.debug('Returning data in native zarr format')
             return _get_zarr_data(data)
@@ -326,7 +330,7 @@ class XarrayProvider(BaseProvider):
             tmp_max = data.coords[self.y_field].values
 
         if tmp_min > tmp_max:
-            LOGGER.debug('Reversing direction of {}'.format(self.y_field))
+            LOGGER.debug(f'Reversing direction of {self.y_field}')
             miny = tmp_max
             maxy = tmp_min
 
@@ -468,9 +472,7 @@ class XarrayProvider(BaseProvider):
         }
 
         if 'crs' in self._data.variables.keys():
-            properties['bbox_crs'] = '{}/{}'.format(
-                'http://www.opengis.net/def/crs/OGC/1.3/',
-                self._data.crs.epsg_code)
+            properties['bbox_crs'] = f'http://www.opengis.net/def/crs/OGC/1.3/{self._data.crs.epsg_code}'  # noqa
 
             properties['inverse_flattening'] = self._data.crs.\
                 inverse_flattening
@@ -499,11 +501,11 @@ class XarrayProvider(BaseProvider):
 
         return {
             'id': name,
-            'description': attrs.get('long_name', None),
-            'unit_label': attrs.get('units', None),
-            'unit_symbol': attrs.get('units', None),
+            'description': attrs.get('long_name'),
+            'unit_label': attrs.get('units'),
+            'unit_symbol': attrs.get('units'),
             'observed_property_id': name,
-            'observed_property_name': attrs.get('long_name', None)
+            'observed_property_name': attrs.get('long_name')
         }
 
     def get_time_resolution(self):
@@ -519,7 +521,7 @@ class XarrayProvider(BaseProvider):
             dt = np.array([time_diff.values.astype('timedelta64[{}]'.format(x))
                            for x in ['Y', 'M', 'D', 'h', 'm', 's', 'ms']])
 
-            return str(dt[np.array([x.astype(np.int) for x in dt]) > 0][0])
+            return str(dt[np.array([x.astype(np.int32) for x in dt]) > 0][0])
         else:
             return None
 
@@ -539,7 +541,7 @@ class XarrayProvider(BaseProvider):
             'seconds': int(ms_difference / 1000) % 60
         }
 
-        times = ['{} {}'.format(val, key) for key, val
+        times = [f'{val} {key}' for key, val
                  in time_dict.items() if val > 0]
 
         return ', '.join(times)
@@ -556,9 +558,15 @@ def _to_datetime_string(datetime_obj):
 
     try:
         value = np.datetime_as_string(datetime_obj)
+    except TypeError as err:
+        LOGGER.warning(err)
+        if datetime_obj.microsecond != 0:
+            value = datetime_obj.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
+        else:
+            value = datetime_obj.strftime('%Y-%m-%dT%H:%M:%SZ')
     except Exception as err:
         LOGGER.warning(err)
-        value = datetime_obj.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
+        raise RuntimeError(err)
 
     return value
 
@@ -596,12 +604,17 @@ def _get_zarr_data(data):
        """
 
     tmp_dir = tempfile.TemporaryDirectory().name
-    data.to_zarr('{}zarr.zarr'.format(tmp_dir), mode='w')
-    with zipfile.ZipFile('{}zarr.zarr.zip'.format(tmp_dir),
-                         'w', zipfile.ZIP_DEFLATED) as zipf:
-        _zip_dir('{}zarr.zarr'.format(tmp_dir), zipf, os.getcwd())
-    zip_file = open('{}zarr.zarr.zip'.format(tmp_dir), 'rb')
-    return zip_file.read()
+
+    zarr_data_filename = f'{tmp_dir}zarr.zarr'
+    zarr_zip_filename = f'{tmp_dir}zarr.zarr.zip'
+
+    data.to_zarr(zarr_data_filename, mode='w')
+
+    with zipfile.ZipFile(zarr_zip_filename, 'w', zipfile.ZIP_DEFLATED) as zipf:  # noqa
+        _zip_dir(zarr_data_filename, zipf, os.getcwd())
+
+    with open(zarr_zip_filename, 'rb') as fh:
+        return fh.read()
 
 
 def _convert_float32_to_float64(data):
