@@ -2,9 +2,11 @@
 #
 # Authors: Francesco Bartoli <xbartolone@gmail.com>
 #          Tom Kralidis <tomkralidis@gmail.com>
+#          Abdulazeez Abdulazeez Adeshina <youngestdev@gmail.com>
 #
 # Copyright (c) 2020 Francesco Bartoli
 # Copyright (c) 2022 Tom Kralidis
+# Copyright (c) 2022 Abdulazeez Abdulazeez Adeshina
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -31,13 +33,15 @@
 """ Starlette module providing the route paths to the api"""
 
 import os
+from typing import Union
+from pathlib import Path
 
 import click
 
 from starlette.staticfiles import StaticFiles
 from starlette.applications import Starlette
 from starlette.requests import Request
-from starlette.responses import Response
+from starlette.responses import Response, JSONResponse, HTMLResponse
 import uvicorn
 
 from pygeoapi.api import API
@@ -51,10 +55,15 @@ if 'PYGEOAPI_CONFIG' not in os.environ:
 with open(os.environ.get('PYGEOAPI_CONFIG'), encoding='utf8') as fh:
     CONFIG = yaml_load(fh)
 
-STATIC_DIR = '{}{}static'.format(os.path.dirname(os.path.realpath(__file__)),
-                                 os.sep)
-if 'templates' in CONFIG['server']:
-    STATIC_DIR = CONFIG['server']['templates'].get('static', STATIC_DIR)
+p = Path(__file__)
+
+app = Starlette(debug=True)
+STATIC_DIR = Path(p).parent.resolve() / 'static'
+
+try:
+    STATIC_DIR = Path(CONFIG['server']['templates']['static'])
+except KeyError:
+    pass
 
 app = Starlette()
 app.mount('/static', StaticFiles(directory=STATIC_DIR))
@@ -64,18 +73,21 @@ if CONFIG['server'].get('cors', False):
     from starlette.middleware.cors import CORSMiddleware
     app.add_middleware(CORSMiddleware, allow_origins=['*'])
 
-OGC_SCHEMAS_LOCATION = CONFIG['server'].get('ogc_schemas_location', None)
+try:
+    OGC_SCHEMAS_LOCATION = Path(CONFIG['server']['ogc_schemas_location'])
+except KeyError:
+    OGC_SCHEMAS_LOCATION = None
 
 if (OGC_SCHEMAS_LOCATION is not None and
-        not OGC_SCHEMAS_LOCATION.startswith('http')):
-    if not os.path.exists(OGC_SCHEMAS_LOCATION):
+        not OGC_SCHEMAS_LOCATION.name.startswith('http')):
+    if not OGC_SCHEMAS_LOCATION.exists():
         raise RuntimeError('OGC schemas misconfigured')
     app.mount('/schemas', StaticFiles(directory=OGC_SCHEMAS_LOCATION))
 
 api_ = API(CONFIG)
 
 
-def get_response(result: tuple) -> Response:
+def get_response(result: tuple) -> Union[Response, JSONResponse, HTMLResponse]:
     """
     Creates a Starlette Response object and updates matching headers.
 
@@ -86,7 +98,13 @@ def get_response(result: tuple) -> Response:
     """
 
     headers, status, content = result
-    response = Response(content=content, status_code=status)
+    if headers['Content-Type'] == 'text/html':
+        response = HTMLResponse(content=content, status_code=status)
+    else:
+        if isinstance(content, dict):
+            response = JSONResponse(content, status_code=status)
+        else:
+            response = Response(content, status_code=status)
 
     if headers is not None:
         response.headers.update(headers)
@@ -137,26 +155,8 @@ async def conformance(request: Request):
     return get_response(api_.conformance(request))
 
 
-@app.route('/collections')
-@app.route('/collections/')
-@app.route('/collections/{collection_id}')
-@app.route('/collections/{collection_id}/')
-async def collections(request: Request, collection_id=None):
-    """
-    OGC API collections endpoint
-
-    :param request: Starlette Request instance
-    :param collection_id: collection identifier
-
-    :returns: Starlette HTTP Response
-    """
-    if 'collection_id' in request.path_params:
-        collection_id = request.path_params['collection_id']
-    return get_response(api_.describe_collections(request, collection_id))
-
-
-@app.route('/collections/{collection_id}/queryables')
-@app.route('/collections/{collection_id}/queryables/')
+@app.route('/collections/{collection_id:path}/queryables')
+@app.route('/collections/{collection_id:path}/queryables/')
 async def collection_queryables(request: Request, collection_id=None):
     """
     OGC API collections queryables endpoint
@@ -171,34 +171,55 @@ async def collection_queryables(request: Request, collection_id=None):
     return get_response(api_.get_collection_queryables(request, collection_id))
 
 
-@app.route('/collections/{name}/tiles')
-@app.route('/collections/{name}/tiles/')
-async def get_collection_tiles(request: Request, name=None):
+@app.route('/collections/{collection_id:path}/tiles')
+@app.route('/collections/{collection_id:path}/tiles/')
+async def get_collection_tiles(request: Request, collection_id=None):
     """
     OGC open api collections tiles access point
 
     :param request: Starlette Request instance
-    :param name: identifier of collection name
+    :param collection_id: collection identifier
 
     :returns: Starlette HTTP Response
     """
-    if 'name' in request.path_params:
-        name = request.path_params['name']
-    return get_response(api_.get_collection_tiles(request, name))
+    if 'collection_id' in request.path_params:
+        collection_id = request.path_params['collection_id']
+    return get_response(api_.get_collection_tiles(
+        request, collection_id))
 
 
-@app.route('/collections/{name}/tiles/\
-    {tileMatrixSetId}/{tile_matrix}/{tileRow}/{tileCol}')
-@app.route('/collections/{name}/tiles/\
-    {tileMatrixSetId}/{tile_matrix}/{tileRow}/{tileCol}/')
-async def get_collection_items_tiles(request: Request, name=None,
+@app.route('/collections/{collection_id:path}/tiles/{tileMatrixSetId}')
+@app.route('/collections/{collection_id:path}/tiles/{tileMatrixSetId}/')
+@app.route('/collections/{collection_id:path}/tiles/{tileMatrixSetId}/metadata')  # noqa
+@app.route('/collections/{collection_id:path}/tiles/{tileMatrixSetId}/metadata/')  # noqa
+async def get_collection_tiles_metadata(request: Request, collection_id=None,
+                                        tileMatrixSetId=None):
+    """
+    OGC open api collection tiles service metadata
+
+    :param collection_id: collection identifier
+    :param tileMatrixSetId: identifier of tile matrix set
+
+    :returns: HTTP response
+    """
+    if 'collection_id' in request.path_params:
+        collection_id = request.path_params['collection_id']
+    if 'tileMatrixSetId' in request.path_params:
+        tileMatrixSetId = request.path_params['tileMatrixSetId']
+    return get_response(api_.get_collection_tiles_metadata(
+        request, collection_id, tileMatrixSetId))
+
+
+@app.route('/collections/{collection_id:path}/tiles/{tileMatrixSetId}/{tile_matrix}/{tileRow}/{tileCol}')  # noqa
+@app.route('/collections/{collection_id:path}/tiles/{tileMatrixSetId}/{tile_matrix}/{tileRow}/{tileCol}/')  # noqa
+async def get_collection_items_tiles(request: Request, collection_id=None,
                                      tileMatrixSetId=None, tile_matrix=None,
                                      tileRow=None, tileCol=None):
     """
     OGC open api collection tiles service
 
     :param request: Starlette Request instance
-    :param name: identifier of collection name
+    :param collection_id: collection identifier
     :param tileMatrixSetId: identifier of tile matrix set
     :param tile_matrix: identifier of {z} matrix index
     :param tileRow: identifier of {y} matrix index
@@ -206,8 +227,8 @@ async def get_collection_items_tiles(request: Request, name=None,
 
     :returns: HTTP response
     """
-    if 'name' in request.path_params:
-        name = request.path_params['name']
+    if 'collection_id' in request.path_params:
+        collection_id = request.path_params['collection_id']
     if 'tileMatrixSetId' in request.path_params:
         tileMatrixSetId = request.path_params['tileMatrixSetId']
     if 'tile_matrix' in request.path_params:
@@ -217,13 +238,16 @@ async def get_collection_items_tiles(request: Request, name=None,
     if 'tileCol' in request.path_params:
         tileCol = request.path_params['tileCol']
     return get_response(api_.get_collection_tiles_data(
-        request, name, tileMatrixSetId, tile_matrix, tileRow, tileCol))
+        request, collection_id, tileMatrixSetId,
+        tile_matrix, tileRow, tileCol))
 
 
-@app.route('/collections/{collection_id}/items', methods=['GET', 'POST'])
-@app.route('/collections/{collection_id}/items/', methods=['GET', 'POST'])
-@app.route('/collections/{collection_id}/items/{item_id}')
-@app.route('/collections/{collection_id}/items/{item_id}/')
+@app.route('/collections/{collection_id:path}/items', methods=['GET', 'POST'])
+@app.route('/collections/{collection_id:path}/items/', methods=['GET', 'POST'])
+@app.route('/collections/{collection_id:path}/items/{item_id:path}',
+           methods=['GET', 'PUT', 'DELETE'])
+@app.route('/collections/{collection_id:path}/items/{item_id:path}/',
+           methods=['GET', 'PUT', 'DELETE'])
 async def collection_items(request: Request, collection_id=None, item_id=None):
     """
     OGC API collections items endpoint
@@ -234,6 +258,7 @@ async def collection_items(request: Request, collection_id=None, item_id=None):
 
     :returns: Starlette HTTP Response
     """
+
     if 'collection_id' in request.path_params:
         collection_id = request.path_params['collection_id']
     if 'item_id' in request.path_params:
@@ -243,17 +268,32 @@ async def collection_items(request: Request, collection_id=None, item_id=None):
             return get_response(
                 api_.get_collection_items(
                     request, collection_id))
-        elif request.method == 'POST':  # filter items
-            return get_response(
-                api_.post_collection_items(
-                    request, collection_id))
+        elif request.method == 'POST':  # filter or manage items
+            content_type = request.headers.get('content-type')
+            if content_type is not None:
+                if content_type == 'application/geo+json':
+                    return get_response(
+                        api_.manage_collection_item(request, 'create',
+                                                    collection_id))
+                else:
+                    return get_response(
+                        api_.post_collection_items(request, collection_id))
+
+    elif request.method == 'DELETE':
+        return get_response(
+            api_.manage_collection_item(request, 'delete',
+                                        collection_id, item_id))
+    elif request.method == 'PUT':
+        return get_response(
+            api_.manage_collection_item(request, 'update',
+                                        collection_id, item_id))
     else:
         return get_response(api_.get_collection_item(
             request, collection_id, item_id))
 
 
-@app.route('/collections/{collection_id}/coverage')
-async def collection_coverage(request: Request, collection_id):
+@app.route('/collections/{collection_id:path}/coverage')
+async def collection_coverage(request: Request, collection_id=None):
     """
     OGC API - Coverages coverage endpoint
 
@@ -268,8 +308,8 @@ async def collection_coverage(request: Request, collection_id):
     return get_response(api_.get_collection_coverage(request, collection_id))
 
 
-@app.route('/collections/{collection_id}/coverage/domainset')
-async def collection_coverage_domainset(request: Request, collection_id):
+@app.route('/collections/{collection_id:path}/coverage/domainset')
+async def collection_coverage_domainset(request: Request, collection_id=None):
     """
     OGC API - Coverages coverage domainset endpoint
 
@@ -285,8 +325,8 @@ async def collection_coverage_domainset(request: Request, collection_id):
         request, collection_id))
 
 
-@app.route('/collections/{collection_id}/coverage/rangetype')
-async def collection_coverage_rangetype(request: Request, collection_id):
+@app.route('/collections/{collection_id:path}/coverage/rangetype')
+async def collection_coverage_rangetype(request: Request, collection_id=None):
     """
     OGC API - Coverages coverage rangetype endpoint
 
@@ -295,6 +335,7 @@ async def collection_coverage_rangetype(request: Request, collection_id):
 
     :returns: Starlette HTTP Response
     """
+
     if 'collection_id' in request.path_params:
         collection_id = request.path_params['collection_id']
 
@@ -302,6 +343,27 @@ async def collection_coverage_rangetype(request: Request, collection_id):
         request, collection_id))
 
 
+@app.route('/collections/{collection_id:path}/map')
+@app.route('/collections/{collection_id:path}/styles/{style_id:path}/map')
+async def collection_map(request: Request, collection_id, style_id=None):
+    """
+    OGC API - Maps map render endpoint
+
+    :param collection_id: collection identifier
+    :param style_id: style identifier
+
+    :returns: HTTP response
+    """
+
+    if 'collection_id' in request.path_params:
+        collection_id = request.path_params['collection_id']
+    if 'style_id' in request.path_params:
+        style_id = request.path_params['style_id']
+
+    return get_response(api_.get_collection_map(
+        request, collection_id, style_id))
+
+
 @app.route('/processes')
 @app.route('/processes/')
 @app.route('/processes/{process_id}')
@@ -348,7 +410,7 @@ async def get_jobs(request: Request, job_id=None):
 
 @app.route('/processes/{process_id}/execution', methods=['POST'])
 @app.route('/processes/{process_id}/execution/', methods=['POST'])
-async def execute_process_jobs(request: Request, process_id):
+async def execute_process_jobs(request: Request, process_id=None):
     """
     OGC API - Processes jobs endpoint
 
@@ -407,16 +469,16 @@ async def get_job_result_resource(request: Request,
         request, job_id, resource))
 
 
-@app.route('/collections/{collection_id}/position')
-@app.route('/collections/{collection_id}/area')
-@app.route('/collections/{collection_id}/cube')
-@app.route('/collections/{collection_id}/trajectory')
-@app.route('/collections/{collection_id}/corridor')
-@app.route('/collections/{collection_id}/instances/{instance_id}/position')
-@app.route('/collections/{collection_id}/instances/{instance_id}/area')
-@app.route('/collections/{collection_id}/instances/{instance_id}/cube')
-@app.route('/collections/{collection_id}/instances/{instance_id}/trajectory')
-@app.route('/collections/{collection_id}/instances/{instance_id}/corridor')
+@app.route('/collections/{collection_id:path}/position')
+@app.route('/collections/{collection_id:path}/area')
+@app.route('/collections/{collection_id:path}/cube')
+@app.route('/collections/{collection_id:path}/trajectory')
+@app.route('/collections/{collection_id:path}/corridor')
+@app.route('/collections/{collection_id:path}/instances/{instance_id}/position')  # noqa
+@app.route('/collections/{collection_id:path}/instances/{instance_id}/area')
+@app.route('/collections/{collection_id:path}/instances/{instance_id}/cube')
+@app.route('/collections/{collection_id:path}/instances/{instance_id}/trajectory')  # noqa
+@app.route('/collections/{collection_id:path}/instances/{instance_id}/corridor')  # noqa
 async def get_collection_edr_query(request: Request, collection_id=None, instance_id=None):  # noqa
     """
     OGC EDR API endpoints
@@ -438,6 +500,24 @@ async def get_collection_edr_query(request: Request, collection_id=None, instanc
                                                       instance_id, query_type))
 
 
+@app.route('/collections')
+@app.route('/collections/')
+@app.route('/collections/{collection_id:path}')
+@app.route('/collections/{collection_id:path}/')
+async def collections(request: Request, collection_id=None):
+    """
+    OGC API collections endpoint
+
+    :param request: Starlette Request instance
+    :param collection_id: collection identifier
+
+    :returns: Starlette HTTP Response
+    """
+    if 'collection_id' in request.path_params:
+        collection_id = request.path_params['collection_id']
+    return get_response(api_.describe_collections(request, collection_id))
+
+
 @app.route('/stac')
 async def stac_catalog_root(request: Request):
     """
@@ -472,14 +552,20 @@ def serve(ctx, server=None, debug=False):
     as a uvicorn server. Not recommend for production.
 
     :param server: `string` of server type
-    :param debug: `bool` of whether to run in debug mode
+    :param debug: `bool` of whether to run in debug mode,
+                    default log level is INFO
 
     :returns: void
     """
 
-#    setup_logger(CONFIG['logging'])
+    log_level = 'info'
+    if debug:
+        log_level = 'debug'
     uvicorn.run(
-        app, debug=True,
+        "pygeoapi.starlette_app:app",
+        reload=True,
+        log_level=log_level,
+        loop='asyncio',
         host=api_.config['server']['bind']['host'],
         port=api_.config['server']['bind']['port'])
 
