@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2020 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -34,14 +34,16 @@ from typing import List
 from datetime import date, datetime, time
 from decimal import Decimal
 from enum import Enum
-import io
 import json
 import logging
 import mimetypes
 import os
+from pathlib import Path
 import re
+from typing import Any, IO, Union
 from urllib.request import urlopen
 from urllib.parse import urlparse
+
 from shapely.geometry import Polygon
 
 import dateutil.parser
@@ -58,14 +60,13 @@ LOGGER = logging.getLogger(__name__)
 
 DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
 
-TEMPLATES = '{}{}templates'.format(os.path.dirname(
-    os.path.realpath(__file__)), os.sep)
+TEMPLATES = Path(__file__).parent.resolve() / 'templates'
 
 mimetypes.add_type('text/plain', '.yaml')
 mimetypes.add_type('text/plain', '.yml')
 
 
-def dategetter(date_property, collection):
+def dategetter(date_property: str, collection: dict) -> str:
     """
     Attempts to obtain a date value from a collection.
 
@@ -76,7 +77,7 @@ def dategetter(date_property, collection):
                for an open interval using null)
     """
 
-    value = collection.get(date_property, None)
+    value = collection.get(date_property)
 
     if value is None:
         return None
@@ -84,7 +85,7 @@ def dategetter(date_property, collection):
     return value.isoformat()
 
 
-def get_typed_value(value):
+def get_typed_value(value: str) -> Union[float, int, str]:
     """
     Derive true type from data value
 
@@ -106,7 +107,7 @@ def get_typed_value(value):
     return value2
 
 
-def yaml_load(fh):
+def yaml_load(fh: IO) -> dict:
     """
     serializes a YAML files into a pyyaml object
 
@@ -122,7 +123,8 @@ def yaml_load(fh):
     def path_constructor(loader, node):
         env_var = path_matcher.match(node.value).group(1)
         if env_var not in os.environ:
-            raise EnvironmentError('Undefined environment variable in config')
+            msg = f'Undefined environment variable {env_var} in config'
+            raise EnvironmentError(msg)
         return get_typed_value(os.path.expandvars(node.value))
 
     class EnvVarLoader(yaml.SafeLoader):
@@ -134,7 +136,7 @@ def yaml_load(fh):
     return yaml.load(fh, Loader=EnvVarLoader)
 
 
-def str2bool(value):
+def str2bool(value: Union[bool, str]) -> bool:
     """
     helper function to return Python boolean
     type (source: https://stackoverflow.com/a/715468)
@@ -154,7 +156,7 @@ def str2bool(value):
     return value2
 
 
-def to_json(dict_, pretty=False):
+def to_json(dict_: dict, pretty: bool = False) -> str:
     """
     Serialize dict to json
 
@@ -173,7 +175,7 @@ def to_json(dict_, pretty=False):
                       indent=indent)
 
 
-def format_datetime(value, format_=DATETIME_FORMAT):
+def format_datetime(value: str, format_: str = DATETIME_FORMAT) -> str:
     """
     Parse datetime as ISO 8601 string; re-present it in particular format
     for display in HTML
@@ -190,7 +192,7 @@ def format_datetime(value, format_=DATETIME_FORMAT):
     return dateutil.parser.isoparse(value).strftime(format_)
 
 
-def file_modified_iso8601(filepath):
+def file_modified_iso8601(filepath: Path) -> str:
     """
     Provide a file's ctime in ISO8601
 
@@ -203,7 +205,7 @@ def file_modified_iso8601(filepath):
         os.path.getctime(filepath)).strftime('%Y-%m-%dT%H:%M:%SZ')
 
 
-def human_size(nbytes):
+def human_size(nbytes: int) -> str:
     """
     Provides human readable file size
 
@@ -228,12 +230,12 @@ def human_size(nbytes):
     elif suffixes[i] == 'B':
         return nbytes
     else:
-        f = '{:.1f}'.format(nbytes).rstrip('0').rstrip('.')
+        f = f'{nbytes:.1f}'.rstrip('0').rstrip('.')
 
-    return '{}{}'.format(f, suffixes[i])
+    return f'{f}{suffixes[i]}'
 
 
-def format_duration(start, end=None):
+def format_duration(start: str, end: str = None) -> str:
     """
     Parse a start and (optional) end datetime as ISO 8601 strings, calculate
     the difference, and return that duration as a string.
@@ -243,6 +245,7 @@ def format_duration(start, end=None):
 
     :returns: string
     """
+
     if not isinstance(start, str) or not start.strip():
         return ''
     end = end or start
@@ -250,7 +253,7 @@ def format_duration(start, end=None):
     return str(duration)
 
 
-def get_path_basename(urlpath):
+def get_path_basename(urlpath: str) -> str:
     """
     Helper function to derive file basename
 
@@ -259,14 +262,16 @@ def get_path_basename(urlpath):
     :returns: string of basename of URL path
     """
 
-    return os.path.basename(urlpath)
+    return Path(urlpath).name
 
 
-def json_serial(obj):
+def json_serial(obj: Any) -> str:
     """
     helper function to convert to JSON non-default
     types (source: https://stackoverflow.com/a/22238613)
+
     :param obj: `object` to be evaluated
+
     :returns: JSON non-default type to `str`
     """
 
@@ -281,20 +286,26 @@ def json_serial(obj):
             return base64.b64encode(obj)
     elif isinstance(obj, Decimal):
         return float(obj)
+    elif type(obj).__name__ == 'int64':
+        return int(obj)
+    elif type(obj).__name__ == 'float64':
+        return float(obj)
     elif isinstance(obj, l10n.Locale):
         return l10n.locale2str(obj)
 
-    msg = '{} type {} not serializable'.format(obj, type(obj))
+    msg = f'{obj} type {type(obj)} not serializable'
     LOGGER.error(msg)
     raise TypeError(msg)
 
 
-def is_url(urlstring):
+def is_url(urlstring: str) -> bool:
     """
     Validation function that determines whether a candidate URL should be
     considered a URI. No remote resource is obtained; this does not check
     the existence of any remote resource.
+
     :param urlstring: `str` to be evaluated as candidate URL.
+
     :returns: `bool` of whether the URL looks like a URL.
     """
     try:
@@ -304,7 +315,8 @@ def is_url(urlstring):
         return False
 
 
-def render_j2_template(config, template, data, locale_=None):
+def render_j2_template(config: dict, template: Path,
+                       data: dict, locale_: str = None) -> str:
     """
     render Jinja2 template
 
@@ -320,17 +332,15 @@ def render_j2_template(config, template, data, locale_=None):
     try:
         templates_path = config['server']['templates']['path']
         env = Environment(loader=FileSystemLoader(templates_path),
-                          extensions=['jinja2.ext.i18n',
-                                      'jinja2.ext.autoescape'],
+                          extensions=['jinja2.ext.i18n'],
                           autoescape=select_autoescape(['html', 'xml']))
         custom_templates = True
-        LOGGER.debug('using custom templates: {}'.format(templates_path))
+        LOGGER.debug(f'using custom templates: {templates_path}')
     except (KeyError, TypeError):
         env = Environment(loader=FileSystemLoader(TEMPLATES),
-                          extensions=['jinja2.ext.i18n',
-                                      'jinja2.ext.autoescape'],
+                          extensions=['jinja2.ext.i18n'],
                           autoescape=select_autoescape(['html', 'xml']))
-        LOGGER.debug('using default templates: {}'.format(TEMPLATES))
+        LOGGER.debug(f'using default templates: {TEMPLATES}')
 
     env.filters['to_json'] = to_json
     env.filters['format_datetime'] = format_datetime
@@ -358,15 +368,16 @@ def render_j2_template(config, template, data, locale_=None):
             LOGGER.debug('Custom template not found; using default')
             env = Environment(loader=FileSystemLoader(TEMPLATES),
                               extensions=['jinja2.ext.i18n'])
+            env.install_gettext_translations(translations)
             template = env.get_template(template)
         else:
             raise
 
     return template.render(config=l10n.translate_struct(config, locale_, True),
-                           data=data, version=__version__)
+                           data=data, locale=locale_, version=__version__)
 
 
-def get_mimetype(filename):
+def get_mimetype(filename: str) -> str:
     """
     helper function to return MIME type of a given file
 
@@ -378,7 +389,7 @@ def get_mimetype(filename):
     return mimetypes.guess_type(filename)[0]
 
 
-def get_breadcrumbs(urlpath):
+def get_breadcrumbs(urlpath: str) -> list:
     """
     helper function to make breadcrumbs from a URL path
 
@@ -405,7 +416,7 @@ def get_breadcrumbs(urlpath):
     return links
 
 
-def filter_dict_by_key_value(dict_, key, value):
+def filter_dict_by_key_value(dict_: dict, key: str, value: str) -> dict:
     """
     helper function to filter a dict by a dict key
 
@@ -419,7 +430,7 @@ def filter_dict_by_key_value(dict_, key, value):
     return {k: v for (k, v) in dict_.items() if v[key] == value}
 
 
-def filter_providers_by_type(providers, type):
+def filter_providers_by_type(providers: list, type: str) -> dict:
     """
     helper function to filter a list of providers by type
 
@@ -430,20 +441,20 @@ def filter_providers_by_type(providers, type):
     """
 
     providers_ = {provider['type']: provider for provider in providers}
-    return providers_.get(type, None)
+    return providers_.get(type)
 
 
-def get_provider_by_type(providers, provider_type):
+def get_provider_by_type(providers: list, provider_type: str) -> dict:
     """
     helper function to load a provider by a provider type
 
     :param providers: ``list`` of providers
-    :param provider_type: type of provider (feature)
+    :param provider_type: type of provider (e.g. feature)
 
     :returns: provider based on type
     """
 
-    LOGGER.debug('Searching for provider type {}'.format(provider_type))
+    LOGGER.debug(f'Searching for provider type {provider_type}')
     try:
         p = (next(d for i, d in enumerate(providers)
                   if d['type'] == provider_type))
@@ -453,7 +464,7 @@ def get_provider_by_type(providers, provider_type):
     return p
 
 
-def get_provider_default(providers):
+def get_provider_default(providers: list) -> dict:
     """
     helper function to get a resource's default provider
 
@@ -470,7 +481,7 @@ def get_provider_default(providers):
         LOGGER.debug('no default provider type.  Returning first provider')
         default = providers[0]
 
-    LOGGER.debug('Default provider: {}'.format(default['type']))
+    LOGGER.debug(f"Default provider: {default['type']}")
     return default
 
 
@@ -487,17 +498,16 @@ class JobStatus(Enum):
     dismissed = 'dismissed'
 
 
-def read_data(path):
+def read_data(path: Union[Path, str]) -> Union[bytes, str]:
     """
     helper function to read data (file or networrk)
     """
 
-    LOGGER.debug('Attempting to read {}'.format(path))
-    scheme = urlparse(path).scheme
+    LOGGER.debug(f'Attempting to read {path}')
 
-    if scheme in ['', 'file']:
+    if isinstance(path, Path) or not path.startswith(('http', 's3')):
         LOGGER.debug('local file on disk')
-        with io.open(path, 'rb') as fh:
+        with Path(path).open('rb') as fh:
             return fh.read()
     else:
         LOGGER.debug('network file')
@@ -505,7 +515,7 @@ def read_data(path):
             return r.read()
 
 
-def url_join(*parts):
+def url_join(*parts: list) -> str:
     """
     helper function to join a URL from a number of parts/fragments.
     Implemented because urllib.parse.urljoin strips subpaths from
@@ -521,7 +531,7 @@ def url_join(*parts):
     return '/'.join([p.strip().strip('/') for p in parts])
 
 
-def get_envelope(coords_list: List[List[float]]):
+def get_envelope(coords_list: List[List[float]]) -> list:
     """
     helper function to get the envelope for a given coordinates
     list through the Shapely API.
